#include "ExplorationAlgo.h"

CPUtime ExplorationAlgo::_CPUtime;

void ExplorationAlgo::recurExploration_1( Reconciliation& reconciliation , uint nodeId , bool withMinimaList ){

  const Node* node = _treePair.geneTree().getNode( nodeId );

  // Leaf node of G
  if( node->isLeafNode() ){
    reconciliation.setExtremeMapping( Lowest , node );

    // A Reconciliation is defined
    if( nodeId == 0 ){
      State* state = 0;
      if( _explorationParam.lik() )
	state = new State( _treePair , reconciliation.recVector() , _explorationParam.birthDeathParam() );
      
      if( !withMinimaList )
	reconciliation.computeLocalMin( _explorationStat->likelihoodStat().likSum() , state );

      _explorationStat->update( reconciliation , withMinimaList );

      if( state != 0 ) delete state;

      // Monitoring exploration
      _monitoring.update();
      //      checkReconciliation( reconciliation , -1 );
    }
    else
      recurExploration_1( reconciliation , nodeId - 1 , withMinimaList );
  }

  // Internal node of G
  else{
    reconciliation.setExtremeMapping( Lowest , node );
    while( true ){
      recurExploration_1( reconciliation , nodeId - 1 , withMinimaList );
      if( !reconciliation.isNMC( Slide( node->nodeId() , UP ) ) )
	break;
      else
	reconciliation.moveMappingUpward( node );
    }
  }
  
  reconciliation.clearMapping( node );
}

// Perform a prefix traversal of G:
// Starts at node id = |V(G) - 1|, corresponding to r(G)
// Ends at node id = 0 (corresponding to a leaf of G)
void ExplorationAlgo::exhaustiveExploration_1( bool withMinimaList ){

  Reconciliation reconciliation( _treePair );
  recurExploration_1( reconciliation , _treePair.geneTree().nbOfNodes() - 1 , withMinimaList );

  Exception( _stateSet.size() != _treePair.nbOfRec() , "TreePair::exhaustiveExploration ---> Anormal case" );
}

// Depth = distNMC( rec , recmin )
void ExplorationAlgo::recurExploration_2( const uint depth  , Reconciliation& reconciliation , P_LIST&  PList , 
					  State* state , list<const CladeNode*>::iterator firstNode , 
					  bool withMinimaList , bool firstPass , int parentId){

  // Update ExplorationStat according to the current node
  if( firstPass && state != 0 ){
    // Update the reconciliation with the highest lik
    _explorationStat->updateLikRec( reconciliation );

    // Update likelihood summation
    _explorationStat->updateLikSum( reconciliation );
  }

  else{
    if( _explorationStat->likelihoodStat().likSum() != 0 )
      reconciliation.valuationValuesUpdate().setValue( PostProb , 
						       reconciliation.costValuation( Lik ) / _explorationStat->likelihoodStat().likSum() );

//     if( !withMinimaList )
//       reconciliation.computeLocalMin( _explorationStat->likelihoodStat().likSum() , state );
    
    // Define nmc distances with: MinRec , RealRec, LikRec
    reconciliation.valuationValuesUpdate().setNMCDist( MinRec  , depth );

    if( _explorationStat->reconciliationNameIsDefined( RealRec ) )
      reconciliation.valuationValuesUpdate().setNMCDist( RealRec , distNMC( *( _explorationStat->reconciliationName( RealRec ) ), reconciliation ) );

    if( _explorationStat->reconciliationNameIsDefined( LikRec ) )
      reconciliation.valuationValuesUpdate().setNMCDist( LikRec , distNMC( *( _explorationStat->reconciliationName( LikRec ) ), reconciliation ) );

    _explorationStat->update( reconciliation , withMinimaList );
    checkReconciliation( reconciliation , parentId , PList );
  }

  _monitoring.update();

  if( depth == _explorationParam.maxDepth() ){
    PList.moveSListIntoQList( firstNode );
    return;
  }

  // Recursively explore the children of the current node
  
  // u <-- v
  list<const CladeNode*>::iterator node = firstNode;

  // While u != end(P)
  while( node != PList.Q().end() ){
    int parentId = -1;
    if( _explorationParam.checkStateSet() )
      parentId = _stateSet.getStateInfo( reconciliation.recVector() ).id();

    // Perform uNMC( u )
    reconciliation.applyNMC( Slide( (**node).nodeId() , UP ) , 0 , state );

    // Update PList according to u_1 and u_2
    bool isInserted1, isInserted2;
    PList.update( node , (**node).child( LEFT  ) , isInserted1 );
    PList.update( node , (**node).child( RIGHT ) , isInserted2 );

    // If u is still a valid uNMC
    if( reconciliation.isNMC( Slide( (**node).nodeId() , UP ) ) )
      recurExploration_2( depth + 1 , reconciliation , PList , state , node , withMinimaList , firstPass , parentId );

    else{
      PList.update_QS( node );
      recurExploration_2( depth + 1 , reconciliation , PList , state , (++node)-- , withMinimaList , firstPass , parentId );
    }

    // Undo previous lines
    reconciliation.applyNMC( Slide( (**node).nodeId() , DOWN ) , 0 , state );

    PList.undoUpdate( node , (**node).child( LEFT  ) , isInserted1 );
    PList.undoUpdate( node , (**node).child( RIGHT ) , isInserted2 );

    node++;
  }
}

void ExplorationAlgo::exhaustiveExploration_2( bool withMinimaList , bool firstPass ){
  ExplorationAlgo::_CPUtime.startClock();

  // rec <-- recmin
  Reconciliation rec( _treePair.extremeRec( Lowest ) );

  State* state = 0;
  if( _explorationParam.lik() ){
    state = new State( _treePair, rec.recVector() , _explorationParam.birthDeathParam() );
    rec.valuationValuesUpdate().setValue( Lik , exp( state->logL() ) );
  }

  // P <-- P(recmin)
  list<const CladeNode*> slideList;
  rec.compute_uNMC( slideList );
  P_LIST PList( _treePair.geneTree() , slideList );

  // v <-- front( P ) and RecurExplore(v)
  list<const CladeNode*>& tmpList = const_cast< list<const CladeNode*>& > ( PList.Q() );

  // Start at depth = 0
  recurExploration_2( 0 , rec , PList , state , tmpList.begin() , withMinimaList , firstPass , -1 );

  if( !firstPass && _explorationParam.checkStateSet() ){
    string errorMSG = "TreePair::exhaustiveExploration ---> Anormal case: ";
    errorMSG += itos( _stateSet.size() ) + " != ";
    errorMSG += itos( _treePair.nbOfRec() );
    Exception( _stateSet.size() != _treePair.nbOfRec() , errorMSG );
  }

  if( state != 0 )
    delete state;

  ExplorationAlgo::_CPUtime.stopClock();
}


//////////////////////////////////////////////
// Debug methods

void ExplorationAlgo::checkReconciliation( const Reconciliation& reconciliation , int parentId , const P_LIST&  PList ){
  if( _explorationParam.checkStateSet() ){

    //  Check reconciliation
    Exception( !reconciliation.isValid()                                                   , "TreePair::checkReconciliation ----> reconciliation is not valid" );
//     Exception( reconciliation.computeCost( Lost )  != reconciliation.costValuation( Lost ) , "TreePair::checkReconciliation ----> error in lost cost" );
//     Exception( reconciliation.computeCost( Dup  )  != reconciliation.costValuation( Dup )  , "TreePair::checkReconciliation   ----> error in dup cost" );
//     Exception( reconciliation.computeCost( Mut  )  != reconciliation.costValuation( Mut )  , "TreePair::checkReconciliation   ----> error in mut cost" );
    Exception( _stateSet.objExists( reconciliation.recVector() )                           , "TreePair::checkReconciliation ----> reconciliation already exists" );
  
    StateInfo* stateInfo = _stateSet.addState( reconciliation.recVector() );
    stateInfo->setParentId( parentId );

    if( _explorationStat->likelihoodStat().likSum() != 0 )
      stateInfo->setPostProb( reconciliation.costValuation( PostProb ) );
  }
}

void ExplorationAlgo::computePostProb(){
  if( _treePair.nbOfRec() != 0 )
    exhaustiveExploration_2( false , true );
}


void ExplorationAlgo::explore(){
  if( _explorationParam.prime() )
    _treePair.extremeRec( Lowest ).writeToPrimeFile( "geneTree.prime" );

  if( _treePair.nbOfRec() != 0 && _explorationParam.performExploration() ){
    exhaustiveExploration_2( false , false );

//     if( _explorationStat->moreThanOneMinima() ){
//       _explorationStat->outputMinimas( "exploration.minimas" );
//       _explorationStat->clearDistribNMC();
//       exhaustiveExploration( true );
//     }

    _explorationParam.resultsStream()<< *_explorationStat;
  }

  //  cerr<<_treePair;

  // Output results
  _explorationParam.resultsStream()<<"\nGeneFam " << _explorationParam.geneFam() << _treePair <<endl;

  _explorationParam.resultsStream()<< "\n\nCheck Stat Set Option: " << btoc( _explorationParam.checkStateSet() );
  _explorationParam.resultsStream()<< "\n\tState Set size: " << _stateSet.size() <<endl;

  _explorationParam.resultsStream()<< "\nAll reconciliations";

  _stateSet.writeAllState( _explorationParam.resultsStream() , _treePair , _explorationParam.birthDeathParam() , 0 );

  if( _explorationParam.maxDepth() != lastUint )
    _explorationParam.resultsStream()<<"\nMax depth\t" << _explorationParam.maxDepth();

}

void ExplorationAlgo::resetMonitor(){
  _monitoring.initialize( _treePair.nbOfRec() , _explorationParam.outputStep() , LOG_FILE + "2" );
}

ExplorationAlgo::ExplorationAlgo( ExplorationParam& explorationParam ):
  _treePair( explorationParam.paramSpeciesTree() , explorationParam.paramGeneTree() ),
  _explorationParam( explorationParam ),
  _stateSet( _treePair ){

  // Initialize MathClass
  // The 1st parameter is used to define the size of the array Factorial
  // Factorial function F(n) is used in SlicedSubtree
  // n <= maximal number of leaves of any Sliced Subtrees
  uint seed = MathClass::genSeed();
  MathClass::initialize( _treePair.extremeRec( Highest ).computeMaximalNbOfCovering() , 
			 seed );


  if( _treePair.nbOfRec() != 0 && _explorationParam.inputRec() )
      _explorationParam.defineReconciliation( _treePair );

  if( _explorationParam.maxDepth() == lastUint )
    _explorationParam.setMaxDepth( _treePair.diameter() );

  _explorationStat = new ExplorationStat( _treePair , _explorationParam );

  _monitoring.initialize( _treePair.nbOfRec() ,  _explorationParam.outputStep() , LOG_FILE + "1" );
  _stateSet.initialize( _treePair.speciesTree().nbOfNodes() +  _treePair.speciesTree().nbOfEdges() ,  _treePair.geneTree().nbOfNodes() );
}

ExplorationAlgo::~ExplorationAlgo(){
  delete _explorationStat;
}
