/***************************************************************************
			ExplorationAlgo.h
		-------------------
	begin	 : Fri Jul  4 11:28:14 EDT 2008
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef EXPLORATIONALGO_H
#define EXPLORATIONALGO_H

//! This class defines the base structure for a ExplorationAlgo object.
/**
*@author Jean-Philippe Doyon
*/

#include "StdClass.h"
#include "toolBox.h"
#include "TreePair.h"
#include "MathClass.h"
#include "ExplorationParam.h"
#include "ExplorationStat.h"
#include "Reconciliation.h"
#include "State.h"
#include "Monitoring.h"
#include "StateSet.h"
#include "P_LIST.h"

class ExplorationAlgo{

 private:
  const TreePair _treePair;
  ExplorationParam& _explorationParam;
  ExplorationStat* _explorationStat;
  Monitoring _monitoring;
  StateSet _stateSet;

  static CPUtime _CPUtime;

 public:

  const static CPUtime& cpuTime() {return _CPUtime;}

  ExplorationAlgo( ExplorationParam& explorationParam );

  void computePostProb();
  void explore();

  void resetMonitor();

  const Monitoring& monitoring() const { return _monitoring; }

  void printResults() const;

  ~ExplorationAlgo();

 private:

  // Two exhaustive exploration algorithms based on a rooted tree

  // V.1: The root is an empty reconciliation and the leaves correspond to the reconciliations
  void recurExploration_1( Reconciliation& reconciliation , uint nodeId , bool withMinimaList );
  void exhaustiveExploration_1( bool withMinimaList );

  // V.2: The spanning tree of the space of reconciliations (see Doyon et al. 2008)
  void recurExploration_2( const uint depth , Reconciliation& reconciliation , P_LIST&  PList ,
			   State* state     , list<const CladeNode*>::iterator firstNode , 
			   bool withMinimaList , bool firstPass , int parentId );

  void exhaustiveExploration_2( bool withMinimaList , bool firstPass );

  void checkReconciliation( const Reconciliation& reconciliation  , int parentId , const P_LIST&  PList );


};

#endif
