/***************************************************************************
			ExplorationParam.h
		-------------------
	begin	 : Wed Apr 23 10:43:06 EDT 2008
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef EXPLORATIONPARAM_H
#define EXPLORATIONPARAM_H

//! This class defines the base structure for a ExplorationParam object.
/**
*@author Jean-Philippe Doyon
*/

#include "ParamSpeciesTree.h"
#include "ParamGeneTree.h"

#include "BirthDeathParam.h"
#include "OrthoParam.h"
#include "Reconciliation.h"

//const string logFileName = ".explorationLog";

class ExplorationParam{

  friend std::ostream& operator << (std::ostream& output, const ExplorationParam& explorationParam );

 private:

  ofstream _resultsStream;

  ParamSpeciesTree _paramSpeciesTree;
  ParamGeneTree _paramGeneTree;

  BirthDeathParam _birthDeathParam;

  // See RandomData.h for semantic
  string _realRecString;

  // The reconciliation induced by the Birth-And-Death
  Reconciliation* _reconciliation;

  ////////////////////////////
  // Integer Options

  // The max dist = max depth of the exploration tree
  uint _maxDepth;
  // Each n steps, write to log files
  uint _outputStep;

  ////////////////////////////
  // Boolean Options

  bool _performExploration;
  bool _prime;
  bool _lik;
  bool _inputRec;
  bool _computeProb;
  bool _checkStateSet;
  bool _computeMappingProb;


 public:

  string resultsFileName() const { return "exploration.results"; }

  uint geneFam() const { return 0; }

  ofstream& resultsStream() { return _resultsStream; }

  ExplorationParam( const string& speciesTreeFile , const string& geneTreeFile , const string& edgeValuesFile , const string& optionString );

  ExplorationParam( const string& speciesTreeFile , const string& edgeValuesFile , const string& optionString , const string& geneTreeString , const string& realRecString );

  // Access to Param Gene/Species Tree
  const ParamSpeciesTree& paramSpeciesTree() const { return _paramSpeciesTree; }
  const ParamGeneTree& paramGeneTree()       const { return _paramGeneTree; }
  const BirthDeathParam& birthDeathParam()   const { return _birthDeathParam; }

  string realRecString() const { return _realRecString; }


  // Access to Integer Options

  uint maxDepth() const { return _maxDepth; }
  void setMaxDepth( uint depth ) { _maxDepth = depth; }

  uint outputStep() const { return _outputStep; }

  // Access to Boolean Options
  bool performExploration() const { return _performExploration; }
  bool prime()              const { return _prime; }
  bool lik()                const { return _lik; }
  bool inputRec()           const { return _inputRec; }
  bool computeProb()        const { return _computeProb && _lik; }
  bool checkStateSet()      const { return _checkStateSet; }
  bool computeMappingProb() const { return _computeMappingProb; }
  

  void defineReconciliation( const TreePair& treePair ){
    _reconciliation = new Reconciliation( treePair , _realRecString );
  }

  const Reconciliation* reconciliation() const { 
    Exception( _reconciliation == 0 , "ExplorationParam::reconciliation() --> is not defined" );
    return _reconciliation; 
  }

  ~ExplorationParam();


  static char getOptionChar( Option option );

  static string help();

 private:
  
  void initialize( const string& speciesTreeFile , const string& edgeValuesFile , const string& optionString , const string& geneTreeString , const string& realRecString );
  string getOptionValue( string optionString , Option option );
};

#endif
