#include "P_LIST.h"

P_LIST::P_LIST( const GeneTree& geneTree , const list<const CladeNode*>& slideList ):
  _Qlist( slideList ), 
  _containsNode   ( geneTree.nbOfNodes() , false ),
  _iteratorInQList( geneTree.nbOfNodes() , NULL_PTR ){

  for( list<const CladeNode*>::iterator node = _Qlist.begin(); node != _Qlist.end(); node++ )
    _containsNode[ (**node).nodeId() ] = true;
}

void P_LIST::update( list<const CladeNode*>::iterator node , const Node* child , bool& isInserted ){
  isInserted = false;

  // If child \notin P & child \notin L(G)
  if( !_containsNode[ child->nodeId() ] && !child->isLeafNode() ){
    isInserted = true;
    _containsNode[ child->nodeId() ] = true;
    // u_1
    if( child->isLeftChild() )
      _Qlist.insert( ++node , (const CladeNode*) (child) );
    // u_2
    else
      _Slist.push_front( (const CladeNode*) (child) );
  }
}

// void P_LIST::undoUpdate( list<const CladeNode*>::iterator node , const Node* child , bool isInserted ){
//   if( isInserted ){
    
//   }
// }

void P_LIST::update_QS( list<const CladeNode*>::iterator node ){
  if( _Slist.empty() )
    return;

  // x <-- the node immediately after u in Q
  node++;

  // y <-- the first node of S
  const CladeNode* y = _Slist.front();

  if( node == _Qlist.end() || (**node).prefixId() > y->prefixId() ){
    list<const CladeNode*>::iterator pos = _Qlist.insert( node , y );
    _Slist.pop_front();
    setIteratorInPList( y , pos );
  }
}

void P_LIST::moveSListIntoQList( list<const CladeNode*>::iterator Qiterator ){
  while( !_Slist.empty() ){
    // Remove first node of Slist
    const CladeNode* Snode = _Slist.front(); 
    _Slist.pop_front();

    // Compute its position into Qlist
    while( Qiterator != _Qlist.end() && (**Qiterator).prefixId() < Snode->prefixId() )
      Qiterator++;

    // Insert it into Qlist
    list<const CladeNode*>::iterator pos = _Qlist.insert( Qiterator , Snode );
    setIteratorInPList( Snode , pos );
  }
}

void P_LIST::undoUpdate( list<const CladeNode*>::iterator node , const Node* child , bool isInserted ){
  if( isInserted ){
    _containsNode[ child->nodeId() ] = false;

    list<const CladeNode*>::iterator pos;
    // Child u_1
    if( child->isLeftChild() ){
      pos = ++node;
      Exception( *pos != (const CladeNode*) (child) , "P_LIST::undoUpdate ---> *pos != child" );
      _Qlist.erase( pos );

    }

    // Child u_2 
    else{
      // u_2 \in Q
      if( QlistContains( child ) ){
	pos = iteratorInQList( child );
	_iteratorInQList[ child->nodeId() ] = NULL_PTR;
	Exception( *pos != (const CladeNode*) (child) , "P_LIST::undoUpdate ---> *pos != child" );
	_Qlist.erase( pos );
      }
      // Otherwise, we are in the context of a partial exploration of the recSpace
      // u_2 is located at the front of S
      else{
	Exception( _Slist.empty() , "P_LIST::undoUpdate ---> Slist is empty" );
	Exception( _Slist.front()->nodeId() != child->nodeId() , "P_LIST::undoUpdate ---> u_2 is not located at the front of Slist" );
	_Slist.pop_front();
      }
    }
  }
}

std::ostream& operator << ( std::ostream& output , const P_LIST& pList ){
  //  output<< pList.size();
  
  output<<"Q(";
  for( list<const CladeNode*>::const_iterator node = pList.Q().begin(); node != pList.Q().end(); node++ )
    output<<" " << (**node).prefixId();
  output<<")";

  output<<"\tS(";
  for( list<const CladeNode*>::const_iterator node = pList.S().begin(); node != pList.S().end(); node++ )
    output<<" " << (**node).prefixId();
  output<<")";

  output<<"\tIte(";
  for( uint i = 0; i < pList.iteratorInQListVector().size(); i++ ){
    if(  pList.iteratorInQListVector()[i] != NULL_PTR ){
      output<< i << "[" << (**(  pList.iteratorInQListVector()[i] )).prefixId() << "]";
    }
  }
  output<<")";

  output<<endl;
  return output;
}
