#ifndef PLIST_H
#define PLIST_H

#include "StdClass.h"
#include "Reconciliation.h"

const list<const CladeNode*>::iterator NULL_PTR = ( list<const CladeNode*>::iterator ) (0);

class P_LIST{

  friend std::ostream& operator << ( std::ostream& output , const P_LIST& pList );

 private:

  // Q union S = P

  // Q inter S = emptyset
  // Q: left child and r(G)
  list<const CladeNode*> _Qlist;
  // S: right child
  list<const CladeNode*> _Slist;

  // u \in P <---> containsNode[ id(u) ] == true
  vector<uint> _containsNode;

  // For any u_2 \in Q \ P(alpha_min):
  //        _iteratorInQList[ id(u_2) ] is the iterator of u_2 in Q
  vector< list<const CladeNode*>::iterator > _iteratorInQList;
  
 public:

  // Used for P <-- P(recmin) at the root of the reconciliation tree
  P_LIST( const GeneTree& geneTree , const list<const CladeNode*>& slideList );

  void update    ( list<const CladeNode*>::iterator node , const Node* child , bool& isInserted );
  void undoUpdate( list<const CladeNode*>::iterator node , const Node* child , bool isInserted );

  void update_QS( list<const CladeNode*>::iterator node );

  // Move each node of S into Q according to the prefix id and after Qiterator
  // Used when the maximal depth of the partial exploration is reached
  void moveSListIntoQList( list<const CladeNode*>::iterator Qiterator );

  const list<const CladeNode*>& Q() const { return _Qlist; }
  const list<const CladeNode*>& S() const { return _Slist; }

  uint size() const { return _Qlist.size() + _Slist.size(); }

  bool QlistContains( const Node* node ){
    return _iteratorInQList[ node->nodeId() ] != NULL_PTR;
  }

  const vector< list<const CladeNode*>::iterator >& iteratorInQListVector() const { return  _iteratorInQList; }

  // Iterator In P list methods
  list<const CladeNode*>::iterator iteratorInQList( const Node* node ) const{
    Exception( node->isLeftChild() , "P_LIST::iteratorInQList ---> only for right child " );
    Exception( node->isLeafNode()  , "P_LIST::iteratorInQList ---> only for internal vertices" );

    uint id = node->nodeId();
    Exception( _iteratorInQList[ id ] == NULL_PTR , "P_LIST::iteratorInQList ---> undefined " + itos( node->prefixId() ) );
    return _iteratorInQList[ id ];
  }

  void setIteratorInPList( const Node* node , list<const CladeNode*>::iterator iterator ){
    Exception( node->isLeftChild()  , "P_LIST::setIteratorInPList ---> only for right child " );
    Exception( node->isLeafNode()   , "P_LIST::setIteratorInPList ---> only for internal vertices" );
    Exception( iterator == NULL_PTR , "P_LIST::setIteratorInPList ---> null iterator " );

    uint id = node->nodeId();
    Exception( _iteratorInQList[ id ] != NULL_PTR , "P_LIST::setIteratorInPList ---> already defined" );
    _iteratorInQList[ id ] = iterator;
  }

};

#endif
