#include "ExplorationAlgo.h"

#include "TREE_PAIR_CPUtimes.h"
#include "OBJECT_SET_CPUtimes.h"

const string GENE_TREE     = "geneTree";
const string SPECIES_TREE = "speciesTree";
const string EDGE_VALUES_FILE  = "edgeValues";

void printHelpMessage(){
  cerr <<"\n-------------------------------------------------------------------";
  cerr <<"\nAuthor: Jean-Philippe Doyon";
//  cerr <<"\n\nexploration F speciesTreeFile geneTreeFile edgeValues [Options]";
  cerr <<"\n\nexploration D repository [Options]";
  cerr <<"\n\tRepository contains speciesTree, geneTree, and edgeValues files";  
//  cerr <<"\nOptions:";
  cerr << ExplorationParam::help();
  cerr <<"\n-------------------------------------------------------------------\n\n";
}

void outputCPUtimes( std::ostream& output ){
  output <<"\nExplorationAlgo: " << ExplorationAlgo::cpuTime();

  TREE_PAIR_CPUtimes::outputCPUtimes( output );
  OBJECT_SET_CPUtimes::outputCPUtimes( output );

  output <<"\n";
}

int main(int argc, char* argv[]){

  if( argc == 1 ){
    printHelpMessage();
    return 0;
  } 

  string dirOrFile = string( argv[1] , strlen( argv[1] ) );

  // Print Help Message
  if( dirOrFile != "D" /*&& dirOrFile != "F" */){
    printHelpMessage();
    return 0;
  }

  // With Directory option
  if( dirOrFile == "D" ){
    string dirName     = string( argv[2] , strlen( argv[2] ) );
    string option( computeStringOption( argc , argv , 3 ) );

    ExplorationParam* explorationParam1 = new ExplorationParam( dirName + "/" + SPECIES_TREE ,  dirName + "/" + GENE_TREE , dirName + "/" + EDGE_VALUES_FILE , option );

    // Check if the given G/S pairs is s.t. M( r(G) ) = r(S)
    // If not, complete the gene tree with a fictive extant gene
    ExplorationParam* explorationParam2 = 0;
    TreePair tmpTreePair( explorationParam1->paramSpeciesTree() , explorationParam1->paramGeneTree() , false );
    if( tmpTreePair.consistantRootMapping() ){
      explorationParam2 = explorationParam1;
    }
    else{
      string geneTreeString, realRecString;
      tmpTreePair.generateConsistantGeneTree( explorationParam1->realRecString() , geneTreeString , realRecString );

      explorationParam2 = new ExplorationParam( dirName + "/" + SPECIES_TREE ,  dirName + "/" + EDGE_VALUES_FILE , option , geneTreeString , realRecString );
    }

    // Start CPU time
    CPUtime cpuTime;
    cpuTime.startClock();

    ExplorationAlgo explorationAlgo( *explorationParam2 );
    //    Exception( !explorationParam2->inputRec() , "Only works with input rec: add \"H\" option" );

    if( explorationParam2->computeProb() ){
      explorationAlgo.computePostProb();
      explorationAlgo.resetMonitor();
    }

    explorationAlgo.explore();

    // Stop CPU time and print time
    cpuTime.stopClock();
    explorationParam2->resultsStream() <<"\nCPU time\t" << cpuTime <<endl;

    ofstream timeStream( explorationAlgo.monitoring().logFileName().c_str() , std::ios::out );
    outputCPUtimes( timeStream );
    timeStream.close();


    // Delete explorationParams
    if( explorationParam1 == explorationParam2 )
      delete explorationParam1;
    else{
      delete explorationParam1;
      delete explorationParam2;
    }
  }


  return 0;
}


//   // With Files option
//   if( dirOrFile == "F" ){
//     string speciesTreeFile = string( argv[2] , strlen( argv[2] ) );
//     string geneTreeFile    = string( argv[3] , strlen( argv[3] ) );
//     string edgeValuesFile  = string( argv[4] , strlen( argv[4] ) );

//     string speciesTree = readFirstLine( speciesTreeFile );
//     ifstream iStream( geneTreeFile.c_str() , std::ios::in );
//     string option( computeStringOption( argc , argv , 5 ) );

//     uint geneFam = 0;

//     while( !iStream.eof() ){
//       string geneTree;
//       getline( iStream , geneTree );

//       if( !geneTree.empty() ){
// 	callExploration( resultsFile , option , speciesTree , geneTree , edgeValuesFile , geneFam );
// 	geneFam++;
//       }
//     }

//     iStream.close();
//   }

