#include "Layer.h"
#include "toolBox.h"
#include "StringLayer.h"
#include "StateLayer.h"
#include "SubException.h"

int Layer::_classCounter = 0;

Layer::Layer( uint layerSize , bool isRoot , const Layer* father , uint pos ):
  AbstractObj( father , pos ) , _vector( layerSize , 0 ) , _isRoot( isRoot ){
  _classCounter++;
}

bool Layer::objExists( uint pos ) const{
  Exception( vectorIdOutOfBound( pos , _vector.size() ) , "Layer::objExists ---> layer size undefined" );
  return _vector[ pos ] != 0;
}

const Layer* Layer::createLayer( uint pos , uint layerSize ){
  try{
    if( isStringLayer() )
      _vector[ pos ] = new StringLayer( layerSize , false , (const StringLayer*) (this) , pos );
    else
      _vector[ pos ] = new StateLayer( layerSize , false , pos );

    _count++;
    return (const Layer*) ( _vector[pos] );
  }

  catch(std::bad_alloc exc){
    throw SubException( exc , "Layer::createLayer" , itos( pos ) );
  }
}

const Layer* Layer::getLayer( uint pos ) const{
  return (const Layer*) ( _vector[ pos ] );
}

void Layer::deleteAbstractObj( AbstractObj* abstractObj ){
  Exception( vectorIdOutOfBound( abstractObj->pos() , _vector.size() ) , "Layer::deleteAbstractObj ---> out of bound" );
  Exception( _vector[ abstractObj->pos() ] != abstractObj              , "Layer::deleteAbstractObj ---> incompatible positions" );

  _vector[ abstractObj->pos() ] = 0;
  delete abstractObj;
  _count--;
}


void Layer::insertAbstractObj( list< AbstractObj* >& listAbstractObj ) const{
  for( int i = 0; i < _vector.size(); i++ )
    if( _vector[i] != 0 )
      listAbstractObj.push_back( _vector[i] );
}
