#include "ObjectSet.h"
#include "toolBox.h"


void ObjectSet::objectSetClear(){
  // Delete all ObjInfo
  for( list< const ObjInfo* >::iterator iter = _objInfoList.begin(); iter != _objInfoList.end(); iter++ )
    delete ( const_cast<ObjInfo*> (*iter) );

  _objInfoList.clear();

  // Delete all Layers
  for( list< const Layer* >::iterator iter = _layerList.begin(); iter != _layerList.end(); iter++ )
    delete ( const_cast<Layer*> (*iter) );

  _layerList.clear();
}

void ObjectSet::deleteAll(){
  if( _rootLayer != 0 ){
    list< AbstractObj* > listAbstractObj;
    listAbstractObj.push_back( _rootLayer );

    while( !listAbstractObj.empty() ){
      AbstractObj* current = listAbstractObj.front();
      listAbstractObj.pop_front();

      if( current->isALayer() )
	( (Layer*) (current) )->insertAbstractObj( listAbstractObj );

      delete current;
    }
  }
}

const ObjInfo& ObjectSet::getObjInfo( const vector<uint>& posVector ) const{
  // Traverse the whole Layer Tree architecture and return the corresponding ObjInfo
  Layer* currentLayer = _rootLayer;
  int i;
  for( i = 0; i < posVector.size() - 1; i++ ){
    Exception( !currentLayer->objExists( posVector[i] ) , "ObjectSet::getObjInfo ---> the object does not exist: " );
    currentLayer = const_cast<Layer*> ( currentLayer->getLayer( posVector[i] ) );
  }

  Exception( !currentLayer->objExists( posVector[i] ) , "ObjectSet::getObjInfo ---> the object does not exist: " );
  return *( currentLayer->getObjInfo( posVector[i]) );
}

const Layer* ObjectSet::retrieveLeafLayer( const vector<uint>& posVector ){
  // Reach the last layer and create new ones when needed

  try{
    Layer* currentLayer = _rootLayer;
    for( int i = 0; i < posVector.size() - 1; i++ ){
      if( !currentLayer->objExists( posVector[i] ) ){
	const Layer* newLayer = currentLayer->createLayer( posVector[i] , _layerSize );
	_layerList.push_back( newLayer );
      }

      currentLayer = const_cast<Layer*> ( currentLayer->getLayer( posVector[i] ) );
    }
    return currentLayer;
  }

  catch( SubException subExc ){
    subExc.addFunDesc( "ObjectSet::retrieveLeafLayer" );
    throw subExc;
  }
}

bool ObjectSet::objExists( const vector<uint>& posVector ) const{
  // Traverse the whole Layer Tree architecture
  Layer* currentLayer = _rootLayer;
  int i;
  for( i = 0; i < posVector.size() - 1; i++ )
    if( currentLayer->objExists( posVector[i] ) )
      currentLayer = const_cast<Layer*> ( currentLayer->getLayer( posVector[i] ) );
    else
      return false;

  return currentLayer->objExists( posVector[i] );
}
