#include "SpaceGraph.h"

SpaceGraph::~SpaceGraph(){
  // Delete nodes
  for( list< SpaceNode* >::iterator node = _nodeList.begin(); node != _nodeList.end(); node++ )
    delete *node;
}

void SpaceGraph::addNode( const StateInfo* stateInfo ){
  if( _nodeVector[ stateInfo->id() ] == 0 ){
    _nodeVector[ stateInfo->id() ] = new SpaceNode( stateInfo , _nodeVector.size() );
    _nodeList.push_back( _nodeVector[ stateInfo->id() ] );
  }
}

void SpaceGraph::addEdge( const StateInfo* stateInfo1 , const StateInfo* stateInfo2 , bool accepted ){
  SpaceNode&       node1 = const_cast<SpaceNode&> ( stateInfoToNode( *stateInfo1 ) );
  const SpaceNode& node2 = stateInfoToNode( *stateInfo2 );

  if( !node1.edgeExists( node2 ) && !node2.edgeExists( node1 ) )
    _nbOfTransitions++;

  node1.addOutgoingEdge( node2 );

  SpaceEdge* spaceEdge = node1.getEdge( node2 );

  if( spaceEdge->totalCount() == 1 )
    _edgeList.push_back( spaceEdge );

  if( accepted )
    spaceEdge->incAcceptedCount();
}

uint SpaceGraph::computeCount( bool total ) const{
  uint count = 0;
  for( list< SpaceEdge* >::const_iterator edge = _edgeList.begin(); edge != _edgeList.end(); edge++ ){
    // Total count
    if( total )
      count += (**edge).totalCount();
    // Accepted count
    else
      count += (**edge).acceptedCount();
  }
  
  return count;
}

void SpaceGraph::writeDotFile( const string& logFileName ) const{
  ofstream dotStream( "dotFile" , std::ios::out );

  // Head of the dot file
  dotStream << "digraph pattern3 {\n";
  dotStream << "\tpage = \"12.5,11\";\n";
  dotStream << "\tsize = \"14.5,12.0\";\n";
  dotStream << "\tlabel = \"Graph \";\n";
  dotStream << "\tfontsize = \"10\";\n";
  dotStream << "\tnodesep = \"0.2\";\n";

  // Write the Nodes
  for( list< SpaceNode* >::const_iterator node = _nodeList.begin(); node != _nodeList.end(); node++ )
    if( *node != 0 )
      dotStream << "\n\t" << (**node).dotString();

  dotStream << "\n\n";

  // Write the edges
  for( list< SpaceEdge* >::const_iterator edge = _edgeList.begin(); edge != _edgeList.end(); edge++ )
    if( (**edge).transProb() != 1 || !inverseEdgeExists( **edge ) )
      dotStream <<"\n\t" << (**edge).dotString();


  // Back of the dot file
  dotStream << "\n}\n";

  dotStream.close();

  string command = "dot -Tps -o ";
  command += truncateExtension( logFileName ) + ".ps ";
  command += "dotFile";

  commandSystem( command );
  commandSystem( "rm -f dotFile" );
}

std::ostream& operator << ( std::ostream& output , const SpaceGraph& spaceGraph ){
  output <<"\n\tNb of nodes: "      << spaceGraph.nbOfNodes();
  output <<"\n\tNb of transitions: " << spaceGraph.nbOfTransitions();
  output <<"\n\tTotal count: "      << spaceGraph.computeCount( true );
  output <<"\n\tAccepted count: "  << spaceGraph.computeCount( false );

  return output;
}

void SpaceGraph::print( std::ostream& output , const OrthoParam& orthoParam , int chainId ) const{
  output<<"\nSpace graph:";
  output<<"\n\tChain id: " << chainId;
  for( list< SpaceNode* >::const_iterator node = _nodeList.begin(); node != _nodeList.end(); node++ ){
    output<< "\n\n";
    (**node).stateInfo()->print( orthoParam , output );;
    output<< endl;
    (**node).printEdges( output , orthoParam , chainId );
  }
  
}

