/***************************************************************************
			SpaceGraph.h
		-------------------
	begin	 : Wed Jan 30 10:01:59 EST 2008
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef SPACEGRAPH_H
#define SPACEGRAPH_H

//! This class defines the base structure for a SpaceGraph object.
/**
*@author Jean-Philippe Doyon
*/

#include "SpaceNode.h"
#include "StdClass.h"

class SpaceGraph{

  friend std::ostream& operator << ( std::ostream& output , const SpaceGraph& spaceGraph );

 private:
  vector< SpaceNode* > _nodeVector;
  list< SpaceNode* > _nodeList;
  list< SpaceEdge* > _edgeList;

  uint _nbOfTransitions;

 public:

  SpaceGraph( uint spaceSize ): _nodeVector( spaceSize , 0 ) , _nbOfTransitions(0) {};

    ~SpaceGraph();

  void addNode( const StateInfo* stateInfo );
  void addEdge( const StateInfo* stateInfo1 , const StateInfo* stateInfo2 , bool accepted );

  const SpaceNode& stateInfoToNode( const StateInfo& stateInfo ) const{
    return *( _nodeVector[ stateInfo.id() ] );
  }

  uint nbOfNodes()       const { return _nodeList.size(); }
  uint nbOfTransitions() const { return _nbOfTransitions; }

  uint computeCount( bool total )      const;

  bool inverseEdgeExists( const SpaceEdge& edge ) const{
    return edge.dest().edgeExists( edge.origin() );
  }

  // Output methods
  void writeDotFile( const string& logFileName ) const;
  void print( std::ostream& output , const OrthoParam& orthoParam , int chainId ) const;
};

#endif
