#include "SpaceNode.h"
#include "MathClass.h"


SpaceNode::~SpaceNode(){
  for( vector< SpaceEdge* >::iterator edge = _edgeVector.begin(); edge != _edgeVector.end(); edge++ )
    delete *edge;
}

SpaceEdge* SpaceNode::getEdge( const SpaceNode& dest ) const{
  return _edgeVector[ dest.id() ];
}

bool SpaceNode::edgeExists( const SpaceNode& dest ) const{
  return _edgeVector[ dest.id() ] != 0;
}

void SpaceNode::addOutgoingEdge( const SpaceNode& dest ){
  if( _edgeVector[ dest.id() ] == 0 )
    _edgeVector[ dest.id() ] = new SpaceEdge( *this , dest );
  else
    _edgeVector[ dest.id() ]->incTotalCount();
}

string SpaceNode::dotString() const{
  string S;

  S += "struct_";
  S += itos( id() );

  S += "[";

  if( _stateInfo->isFirstState() )
    S += "style=bold,";


  S +="label=\"";
  S += _stateInfo->stringId();

//   if( _stateInfo->postProb() != 0 )
//     S += " , " + dtos( _stateInfo->postProb() );
//   else
//     S += " , " + dtos( _stateInfo->approxPostProb() );


  S += "\"];";

  return S;
}

void SpaceNode::printEdges( std::ostream& output , const OrthoParam& orthoParam , int chainId )const{
  for( vector< SpaceEdge* >::const_iterator edge = _edgeVector.begin(); edge != _edgeVector.end(); edge++ )
    if( *edge != 0 ){
      const StateInfo* dest = (**edge).dest().stateInfo();
      double ratio = MathClass::acceptanceProbability( logL() , dest->logL() , orthoParam.paramDoubleValue( TEMP ) , chainId );
      output <<"\t(" << dest->id();
      output << "," << ratio;
      output <<")";
    }
}
