/***************************************************************************
			SpaceNode.h
		-------------------
	begin	 : Wed Jan 30 11:16:40 EST 2008
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef SPACENODE_H
#define SPACENODE_H

//! This class defines the base structure for a SpaceNode object.
/**
*@author Jean-Philippe Doyon
*/

#include "StateInfo.h"
#include "StdClass.h"
#include "SpaceEdge.h"
#include "OrthoParam.h"

class SpaceNode{

 private:
  const StateInfo* _stateInfo;
  vector< SpaceEdge* > _edgeVector;

 public:

  // Constructor
  SpaceNode( const StateInfo* stateInfo , uint spaceSize ):_stateInfo( stateInfo ) , _edgeVector( spaceSize , 0 ){}

  // Destructor
  ~SpaceNode();

  const vector< SpaceEdge* >& edgeVector() const{ return _edgeVector; }

  const StateInfo* stateInfo() const { return _stateInfo; }

  SpaceEdge* getEdge( const StateInfo* stateInfo ) const{ return _edgeVector[ stateInfo->id() ]; }
  SpaceEdge* getEdge( const SpaceNode& dest ) const;

  bool edgeExists( const SpaceNode& dest ) const;

  void addOutgoingEdge( const SpaceNode& dest );

  uint id() const{ return _stateInfo->id(); }
  double logL() const{ return _stateInfo->logL(); }

  // Output methods
  string dotString() const;

  void printEdges( std::ostream& output , const OrthoParam& orthoParam , int chainId ) const;

  

};

#endif
