/***************************************************************************
			StateInfo.h
		-------------------
	begin	 : Fri Jul 27 18:20:24 PDT 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef STATEINFO_H
#define STATEINFO_H

#include "ObjInfo.h"
#include "toolBox.h"
#include "OrthoParam.h"

//! This class defines the base structure for a StateInfo object.
/**
*@author Jean-Philippe Doyon
*/

// Reconciliaton file name
const string REC_FILE_NAME = "reconciliation";

class StateInfo:public ObjInfo{

 private:
  static int _classCounter;
  static uint _currentId;
  const vector<uint> _recVector;
  const string _stringId;
  double _logL;
  uint _id;
  double _postProb;
  double _approxPostProb;

  bool _isFirstState;

  // True <----> logL is the one computed by the whole recursive process defined in Arvestad and al.
  bool _finalLogL;

  // This is on only when CheckStateSet (Check the enumeration algorithm) is on
  // Otherwise, this value is -1
  // Corresponds to the Id in StateInfo (of the parent)
  int _parentId;



 public:

  static int classCounter(){ return _classCounter; }

  // Define ObjInfo as an abstract class
  void bidon(){};

  StateInfo( const vector<uint>& recVector , const string& stringId ):ObjInfo(0,0) , _recVector( recVector ) , _stringId( stringId ){
    _id = _currentId++;
    decCounter(); // StateInfo starts at count = 0 (incremented in the MCMC)
    _logL = 0; // Null value so that it can be used in the logLIsDefined fct
    _finalLogL = false;
    _postProb = 0;
    _approxPostProb = 0;
    _isFirstState = false;
    _parentId = -1;
  }

    ~StateInfo(){}

  // Access methods
  const vector<uint>& recVector() const { return _recVector; }
  const double logL() const { return _logL; }
  uint id() const { return _id; }

  const string& stringId() const { return _stringId; }

  void setLogL( double logL , bool finalLogL ){ 
    _logL = logL; 
    _finalLogL = finalLogL;
  }

  bool logLIsSet() const { return _logL != 0; }
  bool finalLogL() const { return _finalLogL; }

  bool isFirstState() const { return _isFirstState; }
  void setIsFirstState( bool b ) { _isFirstState = b; }

  int parentId() const { return _parentId; }
  void setParentId( int id ) { _parentId = id; }

  // Posterior probabilie
  void setPostProb( double postProb ) { _postProb = postProb; }
  double postProb() const { return _postProb; }

  double approxPostProb() const{ return _approxPostProb; }
  void setApproxPostProb( double approxPostProb ) { _approxPostProb = approxPostProb; }

  double diffPostProb( ) const{
    return fabs( postProb() - approxPostProb() );
  }


  // Output methods
  void writeRecFile( const string& fileName ) const;
  void print( const OrthoParam& orthoParam , std::ostream& output ) const;

};

void retrieveStateInfoValues( const string& inputString , string& stringId , double& postProb );

#endif


