/***************************************************************************
			TreeStringSet.h
		-------------------
	begin	 : Fri Feb  9 17:02:15 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef TREESTRINGSET_H
#define TREESTRINGSET_H

#include "StdClass.h"
#include "StringLayer.h"
#include "ObjectSet.h"

//! This class defines the base structure for a TreeStringSet object.
/**
*@author Jean-Philippe Doyon
*/

const int NULL_CLASS_ID = -1;

class TreeStringSet:public ObjectSet{

  friend std::ostream& operator << (std::ostream& outputStream, const TreeStringSet& set );

 private:
  static int _classCounter;

  int  _upperClassId;
  list<uint> _availableStringId;

 public:

  static int classCounter(){ return _classCounter; }

  // The construction of others attributes are done in initialize method
  TreeStringSet():ObjectSet(){ _classCounter++; }

  // Destructor
  ~TreeStringSet();

  // Access methods
  list< const StringInfo* >& stringInfoList() const { return ( list< const StringInfo* >& ) ( _objInfoList ); }

  void initialize( int upperClassId );
  void clear();

  const StringLayer* rootLayer() const { return (const StringLayer*) _rootLayer; }

  // Remove the treeString from the set if it has a count of 1
  // Return the corresponding String id
  uint removeTreeString( const string& treeString );


  ////////////
  // Version 1
  // Add/remove operations
  const StringInfo* addTreeString( const string& treeString );
  // Between Rootlayer and leafLayer, delete all the ones that are empty
  //    after the removal of StringInfo
  void removeStringInfo( StringInfo* stringInfo );


  ////////////
  // Version 2
  // Add/remove operations on the "StringInfoList" (assuming that the TreeStringSet uses only this structure)
  // Methods used for debug memory problems
  const StringInfo* addTreeString_BASIC( const string& treeString );
  void removeStringInfo_BASIC( StringInfo* stringInfo );


  void decStringInfoCounter( StringInfo* stringInfo );

  uint getStringId( const string& treeString ) const;

  const StringInfo& getStringInfo( const string& treeString ) const;

  StringInfo* getStringInfo( const string& treeString );

  // Boolean method
  bool isTreeStringSet() const { return true; }

  // Define the posVector according to charPos function
  uint charPos( char c , bool root ) const;

 private:
  bool stringIsValid( const string& treeString ) const;
  void stringToPosVector( const string& treeString , vector<uint>& posVector ) const;

  vector<uint> stringToPosVector( const string& treeString ) const{
    vector<uint> posVector;
    stringToPosVector( treeString , posVector );
    return posVector;
  }



};

#endif
