#include "BirthDeathParam.h"

void BirthDeathParam::initialize( const ParamSpeciesTree& paramSpeciesTree , const string& inputFileName , bool withSpeciesId ){
  ifstream stream;
  openFile( inputFileName , stream );

 // Read line "Clade   Lambda  Mu      Length"
  string buffer;
  getline( stream , buffer );

  string cladeName;
  double lambda, mu;
  uint length;
  stream >> cladeName >> lambda >> mu >> length;

  // Define default EdgeValues
  if( cladeName == "All" ){
    _defaultEdgeValues = new EdgeValues( lambda , mu , length );
    stream.close();
    return;
  }
  else
    _defaultEdgeValues = new EdgeValues( LAMBDA , MU , LENGTH );

  while( !stream.eof() ){

    if( !withSpeciesId )
      cladeName = paramSpeciesTree.toSpeciesId( cladeName );

    UintSet clade = stringToSet( cladeName );
    _cladeToValues[ clade ] = new EdgeValues( lambda , mu , length );
    stream >> cladeName >> lambda >> mu >> length;
  }

  stream.close();
}

BirthDeathParam::~BirthDeathParam(){ 
  delete _defaultEdgeValues; 
  for( map< UintSet , EdgeValues* >::iterator itPair = _cladeToValues.begin(); itPair != _cladeToValues.end(); itPair++ )
    delete itPair->second;
}

const EdgeValues& BirthDeathParam::edgeValues( const UintSet& clade ) const{
  if( _cladeToValues.empty() )
    return *_defaultEdgeValues;
  else{
    map< UintSet , EdgeValues* >::const_iterator it = _cladeToValues.find( clade );
    //    Exception( it == _cladeToValues.end() , "BirthDeathParam::edgeValues ---> clade not found: " + setToString( clade ) );
    if( it == _cladeToValues.end() )
      return *_defaultEdgeValues;
    else
      return *( it->second );
  }
}


std::ostream& operator << ( std::ostream& stream , const BirthDeathParam& birthDeathParam ){

  map< UintSet , EdgeValues* >::const_iterator clade;

  stream<< "\nClade\tLambda\tMu\tLength\n";

  for( clade = birthDeathParam.cladeToValues().begin(); clade != birthDeathParam.cladeToValues().end(); clade++ ){
    stream << clade->first;

    stream << "\t";
    stream << clade->second->lambda();

    stream << "\t";
    stream << clade->second->mu();

    stream << "\t";
    stream << clade->second->length();

    stream << endl;
  }

  return stream;
}
