/***************************************************************************
			BirthDeathParam.h
		-------------------
	begin	 : Mon Dec  3 16:46:55 PST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef BIRTHDEATHPARAM_H
#define BIRTHDEATHPARAM_H

//! This class defines the base structure for a BirthDeathParam object.
/**
*@author Jean-Philippe Doyon
*/

#include "StdClass.h"
#include "toolBox.h"
#include "ParamSpeciesTree.h"

// Birth, Death, and Branch length for Species Edge
const double LAMBDA = 0.4;
const double MU     = 0.4;
const uint   LENGTH = 5;

struct EdgeValues{

  friend std::ostream& operator << ( std::ostream& stream , const EdgeValues& edgeValues ){
    stream<< edgeValues.lambda() <<"\t";
    stream<< edgeValues.mu()  <<"\t";
    stream<< edgeValues.length()  <<"\t";

    return stream;
  }
    

private:
  const double _lambda;
  const double _mu;
  const uint _length;

public:

  EdgeValues( double lambda , double mu , uint length ): _lambda( lambda ) , _mu( mu ) , _length( length ){
    //    Exception( _lambda < _mu ,  "EdgeValues::EdgeValues ---> lambda has to be GEQ than mu" );
  }

  double lambda() const { return _lambda; }
  double mu()     const { return _mu; }
  uint length() const { return _length; }
  
};

class BirthDeathParam{

  friend std::ostream& operator << ( std::ostream& stream , const BirthDeathParam& birthDeathParam );

 private:
  map< UintSet , EdgeValues* > _cladeToValues;
  EdgeValues* _defaultEdgeValues;

 public:

  ~BirthDeathParam();

  uint size() const { return _cladeToValues.size(); }

  bool edgeValuesIsDefined( const UintSet& cladeSet ) const{
    map< UintSet , EdgeValues* >::const_iterator it = _cladeToValues.find( cladeSet );
    return it != _cladeToValues.end();
  }

  // Initializer without input file
  void initialize(){ _defaultEdgeValues = new EdgeValues( LAMBDA , MU , LENGTH ); }

  // Initializer with input file
  // withSpeciesId: indicates if the clades are written with the species id
  void initialize( const ParamSpeciesTree& paramSpeciesTree , const string& inputFileName , bool withSpeciesId );

  const map< UintSet , EdgeValues* >& cladeToValues() const { return _cladeToValues; }

  const EdgeValues& edgeValues( const UintSet& clade ) const;
};

#endif
