#include "DST_Parameters.h"

CostType computeCostType( const string& option ){
  if( option.find( DUP_OPTION ) != string::npos )
    return DUP;

  if( option.find( LOST_OPTION ) != string::npos )
    return LOST;

  if( option.find( DUP_LOST_OPTION ) != string::npos )
    return DUP_LOST;

  return NULL_COST;
}

// Constructor
DST_Parameters::DST_Parameters( int argc, char* argv[]  ){

  _paramGeneTree = 0;
  _completeExploration = false;

  // Read input stream
  ifstream inputStream;
  openFile( string( argv[1] ) , inputStream );
  getline( inputStream , _treeString );
  inputStream.close();

  // Create output stream
  string outputFile;
  if( argc == 2 )
    outputFile = string( argv[1] ) + ".bounds";
  else{
    // Compute parameters
    _costType   = computeCostType( string( argv[2] ) );
    _upperBound = stoi( string( argv[3] ) );
    _gap        = stoi( string( argv[4] ) );
    outputFile = string( argv[1] ) + "." + costTypeToString( costType() );
  }

  _outputStream.open( outputFile.c_str() , std::ios::out );

  // Tree with at least two leaves
  if( _treeString.size() <= 1 || _treeString[0] != '(' || _treeString[ _treeString.size() - 1 ] != ';' ){
    _outputStream<<"Anormal tree: \""<< _treeString <<"\"\n";
    _outputStream.close();
    return;
  }

  // Define Param Gene tree
  _paramGeneTree = new ParamGeneTree( _treeString );
}

DST_Parameters::~DST_Parameters(){
  delete _paramGeneTree;
  _outputStream.close(); 
}

string DST_Parameters::help(){

  string s = "branchAndBound treeString C UpperBound Gap";

  s += "\n\nC is:";
  s += "\n\tDup: D";
  s += "\n\tLost: L";
  s += "\n\tMut: M";

  s += "\nUpper bound (int)";
  s += "\nGap (int)";
  return s;
}

string costTypeToString( CostType costType ){
  if( costType == DUP )
    return "dup";

  if( costType == LOST )
    return "loss";

  if( costType == DUP_LOST )
    return "dup_loss";

  if( costType == NULL_COST )
    return "null_cost";
}


std::ostream& operator << (std::ostream& outputStream, const DST_Parameters& param ){
  outputStream<<"\nTree string:" << param.treeString();
  outputStream<<"\n" << costTypeToString( param.costType() );
  outputStream<<"\nUpper bound:" << param.upperBound();
  outputStream<<"\nGap:" << param.gap();
  outputStream<<"\n";
  return outputStream;
}
