/***************************************************************************
			DST_Parameters.h
		-------------------
	begin	 : Wed Feb 28 09:10:05 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef DST_PARAMETERS_H
#define DST_PARAMETERS_H

//! This class defines the base structure for a DST_Parameters object.
/**
*@author Jean-Philippe Doyon
*/

#include "StdClass.h"
#include "toolBox.h"
#include "ParamGeneTree.h"

enum CostType{NULL_COST,DUP,LOST,DUP_LOST};

const char DUP_OPTION = 'D';
const char LOST_OPTION = 'L';
const char DUP_LOST_OPTION = 'M';
const char COMPLETE_EXP_OPTION = 'C';

class DST_Parameters{

  friend std::ostream& operator << (std::ostream& outputStream, const DST_Parameters& param );

  string _treeString;
  CostType _costType;

  ParamGeneTree* _paramGeneTree;
  bool _completeExploration;

  // Maximal accepted gap between optimal cost C* and accepted cost C
  // Gap >= 0
  // C is conserved iff C < C* + Gap 
  uint _gap;
  uint _upperBound;

  ofstream _outputStream;

 public:

  // Constructor
  //             0             1       2  3  4
  // argv: ./branchAndBound treeString C UB Gap 
  // C: L/D/M
  // UB: upperBound (int)
  // Gap: (int)
  DST_Parameters( int argc, char* argv[] );

  ~DST_Parameters();

  bool isNull() const { return _paramGeneTree == 0; }

  // Access method
  CostType costType() const { return _costType; }

  string treeString() const { return _treeString; }

  const ParamGeneTree& paramGeneTree() const { return *_paramGeneTree; }

  uint gap() const{ return _gap; }

  uint upperBound() const { return _upperBound; }

  // Access to Tree Strings
  const string& geneTree1()    const { return _paramGeneTree->treeString1(); }
  const string& geneTree2()    const { return _paramGeneTree->treeString2(); }

  string toSpeciesName( const string& treeString ) const{
    return _paramGeneTree->toSpeciesName( treeString );
  }

  string toSpeciesId( const string& treeString ) const{
    return _paramGeneTree->toSpeciesId( treeString );
  }

  ofstream& outputStream() { return _outputStream; }

  
  uint nbOfSpecies() const { return _paramGeneTree->nbOfSpecies(); }

  bool completeExploration() const { return _completeExploration; }


  // Help String
  static string help();
};

string costTypeToString( CostType costType );
CostType computeCostType( const string& option );

#endif
