#include "ParamTree.h"

ParamTree::ParamTree( string treeString ){ 
  initialize( treeString ); 
}

ParamTree::~ParamTree(){
}

uint ParamTree::specieName2Id( const string& taxaName ) const{
  String_2_Uint_Map::const_iterator iterMap =  _speciesName2Id.find( taxaName );
  Exception( iterMap == _speciesName2Id.end() , "ParamTree::specieName2Id ---> specie name \"" + taxaName + "\" doesn't exists" );

  return iterMap->second;
}

string ParamTree::id2SpeciesName( const string& taxaName  ) const{
  int id = stoi( taxaName );
  Exception( vectorIdOutOfBound( id , _id2SpeciesName.size() ) , "id2SpeciesName ---> id out of bound" );
  return _id2SpeciesName[ id ];
}


string ParamTree::toSpeciesId( const string& originTreeString ) const{
  int i = 0;
  string resultTreeString;

  while( i < originTreeString.size() ){
    if( isalnum( originTreeString[i] ) ){
      string taxaName = readTaxaId( originTreeString , i );
      uint specieId = specieName2Id( taxaName );
      resultTreeString += itos( specieId );
    }
    else
      resultTreeString.append( 1 , originTreeString[i] );

    i++;
  }
  return resultTreeString;
}

void ParamTree::initialize( const string& treeString ){
  _treeString1 = treeString;

  updateSpeciesName2IdMap( treeString );

  _treeString2 = toSpeciesId( treeString );

 // Construct id2SpeciesName vector
 _id2SpeciesName.resize( _speciesName2Id.size() );
 for( String_2_Uint_Map::const_iterator iter = _speciesName2Id.begin();  iter != _speciesName2Id.end(); iter++ ){
   _id2SpeciesName[ iter->second ] = iter->first;
 }
}

// treeString defines a tree with species id
// It can also defines the reconciliation with [speciesId] format
string ParamTree::toSpeciesName( const string& treeString ) const{
  string S;

  int i = 0;
  while( i < treeString.size() ){
    // Skip and write [species id] onto which the next taxa is mapped
    if( treeString[i] == '[' ){
      while( treeString[i] != ']' ){
	S.append( 1 , treeString[i] );
	i++;
      }
      S.append( 1 , treeString[i] );
      i++;
      continue;
    }

    // Read taxa id
    if( isdigit( treeString[i] ) ){
      string taxaName = readTaxaId( treeString , i  );
      i++; // Put on the next char
      S += id2SpeciesName( taxaName );
    }
    else{
      S.append( 1 , treeString[i] );
      i++;
    }
  }

  return S;
}


bool ParamTree::isPrimeFormat() const{
  for( vector<string>::const_iterator S = _id2SpeciesName.begin(); S != _id2SpeciesName.end(); S++ )
    if( !isAlpha( *S ) )
      return false;

  return true;
}

string ParamTree::printSpeciesName2Id() const{
  string S;

  for( String_2_Uint_Map::const_iterator iter = _speciesName2Id.begin();  iter != _speciesName2Id.end(); iter++ )
    S += "\n" + string( iter->first ) + itos( iter->second );

  return S;
}

// External methods
void ParamTree::updateSpeciesName2IdMap( const string& treeString ){
  int i = 0;
  // Taxa name id first value

  while( i < treeString.size() ){
    if( isalnum( treeString[i] ) ){
      string taxaName = readTaxaId( treeString , i );
      if( _speciesName2Id.find( taxaName ) == _speciesName2Id.end() ){
	uint id = _speciesName2Id.size();
	_speciesName2Id[ taxaName ] = id;
      }
    }
    i++;
  }
}
