/***************************************************************************
			ParamTree.h
		-------------------
	begin	 : Tue Nov 20 11:03:24 PST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef PARAMTREE_H
#define PARAMTREE_H

//! This class defines the base structure for a ParamTree object.
/**
*@author Jean-Philippe Doyon
*/

#include "StdClass.h"
#include "toolBox.h"


class ParamTree{

  friend std::ostream& operator << ( std::ostream& output , const ParamTree& paramTree ){
    output <<endl << paramTree.treeString1();
    output <<endl << paramTree.treeString2();    
    return output;
  }

 protected:
  string _treeString1;
  string _treeString2;

  String_2_Uint_Map _speciesName2Id;
  vector<string> _id2SpeciesName;


 public:

  ParamTree( string treeString );

  ParamTree(){}
  
  virtual ~ParamTree();

  void initialize( const string& treeString );

  string toSpeciesName( const string& treeString ) const;

  string toSpeciesId( const string& treeString ) const;

  uint nbOfSpecies() const { return _speciesName2Id.size(); }

  string id2SpeciesName( const string& taxaName ) const;

  uint specieName2Id( const string& taxaName ) const;

  // All species name are alpha num
  // Reconcile, Primetv programs needs that
  bool isPrimeFormat() const;


  bool isNull() const { return _treeString1.empty(); }

  const string& treeString1() const { return _treeString1; }
  const string& treeString2() const { return _treeString2; }

  bool isUnaryTree() const { return isalnum( _treeString1[0] ); }

  bool isBinaryTree() const { return _treeString1[0] =='('; }

  string printSpeciesName2Id() const;

 protected:
  void updateSpeciesName2IdMap( const string& geneTree );


};



#endif
