#ifndef SLIDECLASSES_H
#define SLIDECLASSES_H

enum SlideName{ Spec2Dup, Dup2Spec, Dup2UpDup, Dup2DownDup };

class Slide{

  friend std::ostream& operator << ( std::ostream& output , const Slide& slide ){
    output<< slide.toString();
    return output;
  }

 private:
  int _nodeId; // The node moved by the Slide
  DIRECTION _dir; // The direction of the Slide

 public:
  
  // Default constructor
  Slide(){ _nodeId = -1; }

  // Constructor with values
  Slide( int nodeId , DIRECTION dir ){ 
    _nodeId = nodeId; 
    _dir    = dir;
  }

  // Copy Constructor
  Slide( const Slide& slide ){
    _nodeId = slide.nodeId();
    _dir    = slide.dir();
  }

  // Access methods
  void setValues( int nodeId , DIRECTION dir ){ 
    _nodeId = nodeId; 
    _dir    = dir;
  }

  void unsetValues(){ _nodeId = -1; }
  
  int nodeId() const{
    return _nodeId; 
  }
  
  DIRECTION dir() const{ 
    return _dir; 
  }

  Slide inverse() const{
    return Slide( _nodeId , opposite( _dir ) );
  }

  string dirString() const {
    Exception( !isDefined() , "Slide::dirString() --> is not defined" );
    return ( dir() == UP? "UP": "DOWN" );
  }

  // Information methods
  bool isDefined() const{ return _nodeId != -1; }

  // Operator 
  bool operator == ( const Slide& slide ) const{
    return _nodeId == slide.nodeId() && _dir == slide.dir(); 
  }

  const Slide& operator = ( const Slide& right ){
    setValues( right.nodeId() , right.dir() );    
    return *this;
  }

  string toString() const {
    string s = "(";
    s += itos( _nodeId );
    s += ",";
    s += dirString();
    s += ")";
    return s;
  }

};

// For each Real Gene, there is this bijection:
//       nodeId <---> SlidePos (up/down)
class SlidePos{

 private:
  int _upPos;
  int _downPos;

 public:
  
  // Default constructor
  SlidePos() { _upPos = _downPos = -1; }

  // Access methods
  void setPosition( DIRECTION dir , int pos ){
    if( dir == UP )
      _upPos = pos;
    else
      _downPos = pos;
  }

  void unsetPosition( DIRECTION dir ){ setPosition( dir , -1 ); } 

  int position( DIRECTION dir ) const{ 
    return ( dir == UP? _upPos: _downPos );
  }

  bool isDefined( DIRECTION dir ) const{
    return ( dir == UP? _upPos: _downPos ) != -1;
  }
};

#endif
