#include "Bijection.h"
#include "toolBox.h"

StringBijection::StringBijection( const list<string>& eltList )
  :_eltVector( eltList.begin() , eltList.end() ){
  for( uint i = 0; i < _eltVector.size(); i++ )
    _eltMap[ _eltVector[i] ] = i;
}

const string& StringBijection::elementAt( uint i ) const{
  Exception( vectorIdOutOfBound( i , _eltVector.size() ) , "Bijection::elementAt --- > id out of bound" );
  return _eltVector[i];
}

uint StringBijection::elementId( const string& elt ) const{
  map< string , int >::const_iterator it = _eltMap.find( elt );
  if( it == _eltMap.end() )
    return size();
  else
    return it->second;
}

bool StringBijection::containsElement( const string& elt ) const{
  map< string , int >::const_iterator it = _eltMap.find( elt );
  return it != _eltMap.end();
}



////////////////////////////////////
SetBijection::SetBijection( const list<UintSet>& eltList )
  :_eltVector( eltList.begin() , eltList.end() ){
  for( uint i = 0; i < _eltVector.size(); i++ )
    _eltMap[ _eltVector[i] ] = i;
}

const UintSet& SetBijection::elementAt( uint i ) const{
  Exception( vectorIdOutOfBound( i , _eltVector.size() ) , "Bijection::elementAt --- > id out of bound" );
  return _eltVector[i];
}

uint SetBijection::elementId( const UintSet& elt ) const{
  map< UintSet , int >::const_iterator it = _eltMap.find( elt );

  if( it == _eltMap.end() )
    return size();
  else
    return it->second;
}

bool SetBijection::containsElement( const UintSet& elt ) const{
  map< UintSet , int >::const_iterator it = _eltMap.find( elt );
  return it != _eltMap.end();
}

////////////////////////////////////
std::ostream& operator << (std::ostream& output, const SetBijection& setBijection ){
  for( uint i = 0; i < setBijection.size(); i++ ){
    output<< setBijection.elementAt(i);
    output<<",";
  }

  return output;
}
