#include "MathClass.h"

uint MathClass::_max;
vector<uint> MathClass::_factorial;
vector< vector<uint> > MathClass::_combination;
uint MathClass::_seed;


void MathClass::initialize( uint max , uint seed ){
  // Seed for the Random Generator
  _seed = ( seed == 0? genSeed(): seed );

  srand( _seed );

  _max = max;

  //Define Factiorial Values
  _factorial.resize( _max + 1 );
  _factorial[0] = _factorial[1] = 1;
  for( int i = 2; i <= _max; i++ )
      _factorial[i] = i * _factorial[i-1];

  // Define Combination Values
  // Pascal's rule: (k,n) = (k,n-1) + (k-1,n-1)
  _combination.resize( _max + 1 , vector<uint> (_max + 1) );

  _combination [0] [0] = 1;
  for( int i = 1; i <= _max; i++ ){
    _combination [0] [i] = 1;
    _combination [i] [0] = 0;
  }

  for( int i = 1; i <= _max; i++ )
      for( int j = 1; j <= _max; j++ )
	_combination [i] [j] = _combination [i] [j-1] + _combination [i-1] [j-1];
}

uint MathClass::factorial( uint n ){
  Exception( _factorial.empty() , "MathClass::factorial --->  vector is empty" );
  Exception( n > _max           , "MathClass::factorial ---> value is too large: " + itos( n ) );
  return _factorial[n];
}

uint MathClass::combination( uint k , uint n ){
  Exception( _combination.empty() , "MathClass::combination ---> vector is empty" );
  Exception( k > _max             , "MathClass::combination ---> value is too large: " + itos( k ) );
  Exception( n > _max             , "MathClass::combination ---> value is too large: " + itos( n ) );
  return _combination [k] [n];
}

int MathClass::random( int min , int max ){
  Exception( min > max , "MathClass::random ---> min >= max " );

  if( min == max )
    return min;
  else{
    int scaling = max - min + 1;
    return ( rand() % scaling ) + min;
  }
}

void MathClass::random( int min , int max , int& x , int& y ){
  Exception( min == max , "MathClass::random ---> min = max" );

  // Build vector
  vector< int > values( max - min + 1 , 0 );
  for( int n = min; n <= max; n++ )
    values[ n - min ] = n;

  // Randomly chose i \in {0 , ..., size - 1}
  int i = random( 0 , values.size() - 1 );
  // Define x
  x = values[i];

  if( x != max )
    values[i] = max;

  // Define y
  y = values[ random( 0 , values.size() - 2 ) ];
}

void MathClass::randomIntegers( uint k , uint min , uint max , vector<uint>& intVector ){
  Exception( max - min + 1 < k , "MathClass::randomIntegers ---> max - min + 1 < k" );

  intVector.resize( max - min + 1 );
  int nb = min;
  for( int i = 0; i < intVector.size(); i++ )
    intVector[i] = nb++;

  int lastId = intVector.size() - 1;
  while( lastId + 1 > k ){
    int i = random( 0 , lastId );
    if( i != lastId ){
      intVector[ i ] = intVector[ lastId ];
    }
    lastId--;
  }
  intVector.resize( k );
}

int MathClass::randomNbFromDist( const vector<double>& dist ){
  double probValue = ( (double) ( ( MathClass::random( 1 , 100 ) ) ) ) / 100;

  if( probValue <= dist[0] )
    return 0;

  for( uint i = 1; i < dist.size(); i++ )
    if( distValue( dist , i - 1 ) < probValue && probValue <= distValue( dist , i ) )
      return i;

  string S = "MathClass::randomNbFromDist ---> prob. value not found :";
  S += dtos( probValue , 2 );
  S += "\n dist: " + distToString( dist , 2 );
  S += "\nSize: " + itos( dist.size() );

  Exception( true , S );
}

double MathClass::distValue( const vector<double>& dist , uint i ){
  Exception( isnan( dist[i] ) , "MathClass::distValue ---> dist value is nan" );
  return dist[i];
}

uint MathClass::nbOfRootedTree( uint n ){

  if( n <= 2 )
    return 1;

  uint num = factorial( (2*n) - 3 );
  uint den = pow(2,n-2) * factorial(n-2);

  cerr<<"\nn: " << n;
  cerr<<"\nNum: " << num;
  cerr<<"\nDen: " << den;
  cerr<<"\nN/D: " << num/den;

  return num/den;
}

double MathClass::acceptanceProbability( double logL1 , double logL2 , double temp , int chainId ){
  double heat = pow( 1 + ( chainId * temp ) , -1 );
  double ratio = pow( exp( logL2 ) / exp( logL1 ) , heat );
  return ratio;
}

bool MathClass::acceptProposalState( double logL1 , double logL2 , double temp , int chainId ){
  // Random number between 1 and 10 000
  int n =  MathClass::random( 1 , 10000 );
  double d = (double) (n) / 10000;

  // Compute likelihood ratio
  double ratio  = acceptanceProbability( logL1 , logL2 , temp , chainId );
  bool accepted = ( d <= ratio );

  //  cerr<< setprecision(2)<<"\n" << logL1 <<"\t" << logL2 <<"\t" << ratio << "\t" << accepted;

  return accepted;
}
