/***************************************************************************
			MathClass.h
		-------------------
	begin	 : Thu Mar  1 10:15:49 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef MATHCLASS_H
#define MATHCLASS_H

//! This class defines some Math Functions that need some pretreatment
//! It is an Abstract class, thus it can be instantiated.
//! Use it by
//!   - First, calling MathClass::initialize.
//!   - Second, calling the desired function MathClass::function()

/**
*@author Jean-Philippe Doyon
*/

#include "toolBox.h"
#include "StdClass.h"

const double INVALID_PROB = 2;

class MathClass{

  static uint _max;
  static vector<uint> _factorial;
  static vector< vector<uint> > _combination;
  static uint _seed;

 public:

  // Virtual Destructor
  virtual ~MathClass();

  static uint genSeed(){ return time(NULL) + getpid(); }

  static uint seed(){ return _seed; }

  static void initialize( uint max , uint seed = 0 );

  static uint factorial( uint n );

  static uint combination( uint k , uint n );


  // Return x s.t. min <= x <= max
  static int random( int min , int max );

  // Return x,y s.t. min <= x,y <= max & x != y
  static void random( int min , int max , int& x , int& y );

  // Return k numbers x s.t. min <= x <= max
  // All numbers are differents
  // size of intVector = k
  static void randomIntegers( uint k , uint min , uint max , vector<uint>& intVector );


  static int randomNbFromDist( const vector<double>& dist );

  static double distValue( const vector<double>& dist , uint i );

  // Number of leaf labeled rooted tree for n leaves is
  // (2n - 3)! / ( 2^{n-2} (n-2)! )
  static uint nbOfRootedTree( uint n );

  static double acceptanceProbability( double logL1 , double logL2 , double temp , int chainId );

  static bool acceptProposalState( double logL1 , double logL2 , double temp , int chainId );

  static double mean( double x , double y ){
    return (x + y)/2;
  }

  static double diffRatio( double x , double y ){
    return fabs( x - y ) / fabs( x + y );
  }

  template<class T> static void shuffleList( const list<T>& listT , vector<T>& vectorT ){
    vector<T> temp;
    temp.insert( temp.begin() , listT.begin() , listT.end() );
    vectorT.resize( temp.size() );

    int lastId = temp.size() - 1;
    uint j = 0;
    while( lastId >= 0 ){
      uint i = random( 0 , lastId );
      vectorT[j] = temp[i];
      temp[i] = temp[lastId];
      lastId--;
      j++;      
    }    
  }

  // Define MathClass as an abstract class
  virtual void bidon() = 0;

};

#endif
