#include "setTool.h"
#include "toolBox.h"

void makeUnionSet( const IntSet& set1 , const IntSet& set2 , IntSet& resultSet ){
  resultSet.insert( set1.begin() , set1.end() );
  resultSet.insert( set2.begin() , set2.end() );
}

void makeInterSet( const IntSet& set1 , const IntSet& set2 , IntSet& resultSet ){
  IntSet::const_iterator iter1, iter2;

  for( iter1 = set1.begin(); iter1 != set1.end(); iter1++ ){
    iter2 = set2.find( *iter1 );
    if( iter2 != set2.end() )
      resultSet.insert( *iter2 );
  }
}

bool intersect( const IntSet& set1 , const IntSet& set2 ){
  IntSet inter;
  makeInterSet( set1 , set2 , inter );
  return !inter.empty();
}

void makeUnionSet( const UintSet& set1 , const UintSet& set2 , UintSet& resultSet ){
  resultSet.insert( set1.begin() , set1.end() );
  resultSet.insert( set2.begin() , set2.end() );
}

void makeInterSet( const UintSet& set1 , const UintSet& set2 , UintSet& resultSet ){
  UintSet::const_iterator iter1, iter2;

  for( iter1 = set1.begin(); iter1 != set1.end(); iter1++ ){
    iter2 = set2.find( *iter1 );
    if( iter2 != set2.end() )
      resultSet.insert( *iter2 );
  }
}

void makeDiffSet( const UintSet& set1  , const UintSet& set2 , UintSet& resultSet ){
  for( UintSet::const_iterator iter = set1.begin(); iter != set1.end(); iter++ )
    if( set2.find( *iter ) == set2.end() )
      resultSet.insert( *iter );
}

bool intersect( const UintSet& set1  , const UintSet& set2 ){
  for( UintSet::const_iterator iter = set1.begin(); iter != set1.end(); iter++ )
    if( set2.find( *iter ) != set2.end() )
      return true;
  return false;
}

bool isSubset( const UintSet& set1  , const UintSet& set2 ){
  UintSet Union;
  makeUnionSet( set1 , set2 , Union );
  return Union == set2;
}

string setToString( const UintSet& set ){
  string S;

  UintSet::const_iterator iterSet;
  for( iterSet = set.begin(); iterSet != set.end(); iterSet++ ){
    S += itos( *iterSet );
    if( (++iterSet)-- != set.end() )
      S += " ";
  }
  return S;
}

UintSet stringToSet( const string& genomeString ){
  UintSet set;

  int i = 0;
  while( i < genomeString.size() ){
    if( isdigit( genomeString[i] ) ){
      uint id = stoi( readTaxaId( genomeString , i ) );
      set.insert( id );
    }
    i++;
  }

  return set;
}

string toString( const Uint_Uint_Map& map ){
  string S;
  for( Uint_Uint_Map::const_iterator it = map.begin(); it != map.end(); it++ )
    S += "\n" + itos( it->first ) + "\t" + itos( it->second );

  return S;
}
