#include "stringTool.h"
#include "toolBox.h"

string truncateExtension( const string& fileName ){
  return fileName.substr( 0 , fileName.find('.') );
}


string readTaxaName( const string& treeString , int& pos , bool onlySpecieName ){
  string taxaName;

  while( treeString[ pos ] != ',' &&  treeString[ pos ] != ')'  ){
    if( !( onlySpecieName && isdigit( treeString[ pos ] ) ) ){
      char newChar = treeString[ pos ];
      taxaName.append( 1 , newChar );
    }
    pos++;
  }

  return taxaName;
}

string readTaxaId( const string& treeString , int& pos ){
  string taxaName;

  while( pos < treeString.size() && isalnum( treeString[ pos ] ) ){
    char newChar = treeString[ pos ];
    taxaName.append( 1 , newChar );
    pos++;
  }

  pos--;
  return taxaName;
}

uint readTaxaId( const string& treeString , uint& i ){
  string result;
  while( i < treeString.size() && isalnum( treeString[i] ) ){
    result += treeString[i];
    i++;
  }
  return atoi( result.c_str() );
}

bool isAlpha( const string& S ){
  for( uint i = 0; i < S.size(); i++ )
    if( !isalpha( S[i] ) )
      return false;

  return true;	
}

bool isdigit( const string& S ){
  for( uint i = 0; i < S.size(); i++ )
    if( !isdigit( S[i] ) )
      return false;

  return true;	
}


void skipChar( char c , const string& string , int& pos ){
  while( pos < string.size() && string[pos] == c ) pos++;
}

string readUntilChar( char c , const string& S1 , int& pos ){
  string S2;

  while( pos < S1.size() && S1[pos] != c ){
    S2 += S1[pos];
    pos++;
  }

  return S2;
}

string readAlNum( const string& S , uint& i ){
  string T;

  while( i < S.size() && ( isalnum( S[i] ) || isDecimalPart( S[i] ) ) ){
    T += S[i];
    i++;
  }    

  return T;
}

void skipNonAlNum( const string& S , uint& i ){
  while( i < S.size() && !( isalnum( S[i] ) || isDecimalPart( S[i] ) ) )
    i++;
}

void partitionString( const string& S , vector<string>& stringVector ){
  uint i = 0;
  list<string> stringList;

  while( i < S.size() ){
    skipNonAlNum( S , i );
    if( i < S.size() ){
      string tmp = readAlNum( S , i );
      stringList.push_back( tmp );
    }
  }
  
  stringVector = vector<string> ( stringList.begin() , stringList.end() );
}

bool isSeparator( SEPARATOR separator , char c ){
  if( separator == BLANK )
    return isspace( c );
}

int skipSeparator( SEPARATOR separator , const string& S , int i ){
  int j = i;
  while( j < S.size() && isSeparator( separator , S[j] ) )
    j++;
  return j;
}

int findNextSeparator( SEPARATOR separator , const string& S , int i ){
  int j = i;
  while( j < S.size() && !isSeparator( separator , S[j] ) )
    j++;
  return j;
}

void partitionString( const string& S , vector<string>& stringVector , SEPARATOR separator ){
  uint i = 0;
  list<string> stringList;

  while( i < S.size() ){
    int start = skipSeparator( separator , S , i );
    int end   = findNextSeparator( separator , S , start );
    stringList.push_back( string( S , start , end - start ) );  
    i = end;
  }

  stringVector = vector<string> ( stringList.begin() , stringList.end() );
}

bool charInString( const string& S , char C ){
  return S.find( C ) != string::npos;
}

bool stringInString( const string& S1 , const string& S2 ){
  return S1.find( S2 ) != string::npos; 
}


string retrieveFileName( const string& option , char optionChar ){
  int pos = option.find( optionChar );
  string fileName;

  pos++;
  skipChar( ' ' , option , pos );      // Skip blanks
  fileName = readUntilChar( ' ' , option , pos ); // Read file name

  return fileName;
}
