#include "ArtifGene.h"
#include "CladeSpecies.h"

int ArtifGene::_classCounter = 0;

// Important: this ArtifGene has to be added on the GeneEdge after defining child/father relationship
ArtifGene::ArtifGene( const State& state , const GeneEdge* geneEdge , const CladeSpecies* cladeSpecies ):
  Gene( state ){
  _classCounter++;

  _geneEdge   = geneEdge;
  _dfsVisited = false;

  // Define the reconciliation function (ArtifNode are always covered by a SpeciesNode)
  setCovering( cladeSpecies );

  // Sliced node count of the covering species
  _slicedNodeId = cladeSpecies->slicedNodeCount();
  const_cast<CladeSpecies*> ( cladeSpecies )->incSlicedNodeCount();

  // Add this ArtifNode on the GeneEdge
  _artifGeneIt = ( const_cast<GeneEdge*> (_geneEdge) )->addArtifGene( this );
}

Side ArtifGene::speciesEdgeSide() const{
  Exception( _species == 0 , "ArtifGene::speciesEdgeSide() ---> _species is undefined" );
  const CladeSpecies* cladeSpecies = (const CladeSpecies*) _species;
  if( _geneEdge->isCoveredBy( cladeSpecies->edge(LEFT) ) )
    return LEFT;
  else
    return RIGHT;
}

const SpeciesEdge* ArtifGene::speciesEdge() const{
  const CladeSpecies* cladeSpecies = (const CladeSpecies*) _species;
  if( speciesEdgeSide() == LEFT )
    return cladeSpecies->edge( LEFT );
  else
    return cladeSpecies->edge( RIGHT );
}


void ArtifGene::unSetCovering(){
  Exception( _species == 0 , "ArtifGene::unSetCovering ---> _species == 0" );
  const_cast<Species*> (_species)->removeCoveredGene( this );
  _species = 0;
}


const Gene* ArtifGene::slicedChild() const{
  if( this == _geneEdge->lastArtifGene() )
    return _geneEdge->childNode();
  else{
    list< const ArtifGene* >::iterator temp = _artifGeneIt;
    return *(++temp);
  }
}

const Gene* ArtifGene::slicedFather() const{
  if( this == _geneEdge->firstArtifGene() )
    return _geneEdge->fatherNode();
  else{
    list< const ArtifGene* >::iterator temp = _artifGeneIt;
    return *(--temp);
  }
}

const Gene* ArtifGene::nextDFS_SlicedChild( const SlicedSubtree& slicedSubtree ) const{
  if( slicedSubtree.isTheSlicedRoot( this ) && !_geneEdge->isCoveredBy( slicedSubtree.speciesEdge() ) )
      return 0;

  if( slicedChild()->dfsVisited() )
    return 0;
  else
    return slicedChild();
}

bool ArtifGene::isFirstSlicedChild ( const SlicedSubtree& slicedSubtree  , const Gene* childGene ) const{
  return slicedChild() == childGene;
}

const SlicedSubtree& ArtifGene::uniqueSlicedSubtree() const{
  Exception( _species == 0 , "ArtifGene::uniqueSlicedSubtree() ---> _species is undefined" );
  if( speciesEdgeSide() == LEFT )
    return *_leftSlicedSubtree;
  else
    return *_rightSlicedSubtree;  
}

string ArtifGene::geneString() const{
  return itos( _species->nodeId() ) + "(" + itos( _slicedNodeId ) + ")";
}

 
std::ostream& operator << (std::ostream& outputStream, const ArtifGene& artifGene ){
  outputStream<< "{";
  outputStream<< artifGene.geneString();
  
//  outputStream<< "," << artifGene.slicedFather()->geneString();

  outputStream<< "}";
  return outputStream;
}

