/***************************************************************************
			ArtifGene.h
		-------------------
	begin	 : Fri Oct 27 09:41:59 EDT 2006
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef ARTIFICIALNODE_H
#define ARTIFICIALNODE_H

//! This class defines the base structure for a ArtifGene object.
/**
*@author Jean-Philippe Doyon
*/

// An ArtifGene can only be covered by a species Node 
// They are added on the Gene edges that go through species node without bifurcating (speciation)

#include "StdClass.h"
#include "Gene.h"
#include "GeneEdge.h"

class CladeSpecies;
class SlicedSubtree;

const int ARTIFICIAL_NODE_ID = -1; // For the father node of the artificial edge (species tree)

class ArtifGene:public Gene{

  friend std::ostream& operator << (std::ostream& outputStream, const ArtifGene& artifGene );

 private:

  static int _classCounter;

  const GeneEdge* _geneEdge; // Edge on which the node were added;
  uint _slicedNodeId; // The node id corresponding to ieme sliced node covered by the covering species node
  bool _dfsVisited;

  list< const ArtifGene* >::iterator _artifGeneIt;

 public:

  static int classCounter(){ return _classCounter; }

  // Constructor
  ArtifGene( const State& state , const GeneEdge* geneEdge , const CladeSpecies* cladeSpecies );
  ~ArtifGene(){ _classCounter--; };

  void clearForRedefine(){ Gene::clearForRedefine(); }

  // Access methods
  const GeneEdge* geneEdge() const { return _geneEdge; }
  uint nodeId()              const { return _slicedNodeId; }
  uint slicedNodeId()        const { return _slicedNodeId; }

  Side geneEdgeSide() const { return _geneEdge->isLeftEdge()? LEFT: RIGHT; }

  // The Side of the Species Edge on which the Gene Edge is covered by
  Side speciesEdgeSide() const;
  const SpeciesEdge* speciesEdge() const;

  // DFS visited status
  bool dfsVisited() const     { return _dfsVisited; }
  void setDFS_Visited(bool b) { _dfsVisited = b; }

  // Reconciliation methods
  void unSetCovering();

  // Node type methods
  bool isLeafNode() const  { return false; } // It can not be a leaf node
  bool isArtifNode() const { return true; }
  bool fictiveLeaf() const { return false; }

  // Sliced Subtree methods 
  // It unique child
  const Gene* slicedChild() const;

  const Gene* slicedFather() const;

  const Gene* nextDFS_SlicedChild( const SlicedSubtree& slicedSubtree ) const;
  bool isFirstSlicedChild        ( const SlicedSubtree& slicedSubtree  , const Gene* childGene ) const;

  void removeFromGeneEdge() { const_cast<GeneEdge*> (_geneEdge)->removeArtifGene( this ); }

  const SlicedSubtree& uniqueSlicedSubtree() const;

  // Debug methods
  void checkNode( std::ostream& output ) const{}; // Verify that all node attributes are OK

  // Print methods
  string geneString() const;

};

#endif
