#include "Cell.h"
#include "Node.h"
#include "Edge.h"

uint Cell::id() const{
  if( isANode() )
    return _treePair.cellId( _node );
  else
    return _treePair.cellId( _edge );
}

bool Cell::isARootCell() const{
  if( isANode() )
    return _node->isRootNode();
  else
    return _edge->isRootEdge();
}

Cell* Cell::fatherCell() const{
  Exception( isARootCell() , "Cell::fatherCell() ----> is a root cell" );
  if( isANode() )
    return _treePair.getCell( _node->fatherEdge() );
  else
    return _treePair.getCell( _edge->fatherEdge() );
}

uint Cell::layerId() const{
  if( isANode() )
    return _node->nodeId();
  else
    return _edge->layerId();
}

string Cell::toString() const{
  if( isANode() )
    return itos( _node->nodeId() );
  else
    return _edge->toString();
}

// Two cell A,B \in V(S) \union E(S)
bool Cell::operator <= ( const Cell& cell ) const{
  // A \in V(S)
  if( isANode() ){
    if( cell.isANode() )
      return *( this->node() ) <= *( cell.node() );
    else
      return *( this->node() ) < *( cell.edge()->childNode() );
  }
  // A \in E(S)
  else{
    if( cell.isANode() )
      return *( this->edge()->childNode() ) <= *( cell.node() );
    else
      return *( this->edge()->childNode() ) <= *( cell.edge()->childNode() );
  }
}

const Cell* Cell::getChildNodeCell() const{
  Exception( isANode() , "Cell::getChildNodeCell() ---> is a node" );
  const Node* childNode = edge()->childNode();
  return _treePair.getCell( childNode );
}

const Cell* Cell::getChildEdgeCell( Side side ) const{
  if( isANode() ){
    Exception( node()->isLeafNode() , "Cell::getChildEdgeCell ---> is a leaf node" );
    return _treePair.getCell( ( (const CladeNode*) ( node() ) )->edge( side ) );
  }
  else{
    Exception( edge()->isLeafEdge() , "Cell::getChildEdgeCell ---> is a leaf edge" );
    return _treePair.getCell( edge()->childEdge( side ) );
  }
}

std::ostream& operator << ( std::ostream& output , const Cell& cell ){
  output<< cell.toString();
  return output;
}

uint distCell( const Cell* cell1 , const Cell* cell2 ){

  Exception( !( *cell1 <= *cell2 ) , "Cell::dist --> " + cell1->toString() + " !<= " + cell2->toString() );

  if( cell1->isANode() && cell2->isANode() )
    return cell1->node()->tree().dist( cell1->node() , cell2->node() );

  if( cell1->isANode() && !cell2->isANode() )
    return cell1->node()->tree().dist( cell1->node() , cell2->edge() );

  if( !cell1->isANode() && cell2->isANode() )
    return cell1->edge()->tree().dist( cell1->edge() , cell2->node() );

  if( !cell1->isANode() && !cell2->isANode() )
    return cell1->edge()->tree().dist( cell1->edge() , cell2->edge() );
}

bool areComparable( const Cell* cell1 , const Cell* cell2 ){
  return *cell1 <= *cell2 || *cell2 <= *cell1;
}
