/***************************************************************************
			Cell.h
		-------------------
	begin	 : Wed May 14 11:22:39 EDT 2008
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef CELL_H
#define CELL_H

//! This class defines the base structure for a Cell object.
/**
*@author Jean-Philippe Doyon
*/

class Node;
class Edge;
#include "TreePair.h"

class Cell{

  friend std::ostream& operator << ( std::ostream& output , const Cell& cell );


 private:
  const Node* _node;
  const Edge* _edge;
  const TreePair& _treePair;

 public:

  Cell( const TreePair& treePair , const Node* node ): _treePair( treePair ) , _node( node ) , _edge( 0 ) {}
    
    Cell( const TreePair& treePair , const Edge* edge ): _treePair( treePair ) , _node( 0 ) , _edge( edge ) {}
  
      bool isANode() const{ return _node != 0; }
      uint id() const;

      const TreePair& treePair() const { return _treePair; }

      bool isARootCell() const;
      Cell* fatherCell() const;

      const Node* node() const { return _node; }
      const Edge* edge() const { return _edge; }

      // Only for Edge cell
      const Cell* getChildNodeCell() const;

      const Cell* getChildEdgeCell( Side side ) const;

      uint layerId() const;

      string toString() const;

      string normFormString() const {
	string S = "[";

	if( isANode() )
	  S += itos( _node->nodeId() );
	else
	  S += "E" + itos( _edge->fatherNode()->nodeId() );

	S += "]";
	return S;
      }

      bool operator <= ( const Cell& cell ) const;

      bool operator < ( const Cell& cell ) const{
	return this != &cell && *this <= cell;
      }
};

// Number of nodes u \in V(G) s.t. cell1 < u < cell2
uint distCell( const Cell* cell1 , const Cell* cell2 );

// True <----> cell1 <= cell2 OR cell2 <= cell1
bool areComparable( const Cell* cell1 , const Cell* cell2 );


#endif
