/***************************************************************************
			CladeNode.h
		-------------------
	begin	 : Fri Jun  9 13:58:50 EDT 2006
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef CLADENODE_H
#define CLADENODE_H

//! This class defines the base structure for a CladeNode object.
/**
*@author Jean-Philippe Doyon
*/

#include "Node.h"

enum CHILD_ORDER {LR,RL};

class Edge;
class Reconciliation;

class CladeNode:public Node{

 private:

  static int _classCounter;

  Edge* _leftEdge;
  Edge* _rightEdge;
  UintSet _cladeSet;

 public:

  static int classCounter(){ return _classCounter; };

  // Constructor and Destructor
  CladeNode( uint depth , const Tree& tree , const CladeNode* father = 0 );
  ~CladeNode(){ _classCounter--; }

  // Access methods
  const UintSet& cladeSet() const { return _cladeSet; }

  // Child / edge methods
  const Node* child( Side s ) const; 
  const Edge* edge( Side s ) const;

  bool isBinary() const { return _leftEdge != 0 && _rightEdge != 0; }

  void addChild( Node* child );

  Node* nextDFSChild(CHILD_ORDER order = LR) const;

  void addToCladeSet( const UintSet& setToAdd ) { _cladeSet.insert( setToAdd.begin() , setToAdd.end() ); }

  bool withoutEdge() const { return _leftEdge == 0 && _rightEdge == 0; }

  // Boolean methods
  bool isLeafNode()  const { return false; }
  bool fictiveLeaf() const { return false; }

  // Only for u in V(G) \ L(G)
  // u is a forced duplication <----> M(u) == M(u1) or M(u) == M(u2)
  bool isAForcedDup() const;

  // Cell set method
  void defineCellSet();
  Cell* highestCell() const;
  const Cell* getChildEdgeCell( const Cell* nodeCell ) const;

  // Normal form methods
  void computeNormalFormString( const Reconciliation* reconciliation = 0 );
  bool isFirstInNormForm( const Node* rightNode ) const;

  void interchangeChildren(){ interchange( _leftEdge , _rightEdge ); }

  // Output methods
  string primeString( const Reconciliation& reconciliation ) const;
};

#endif
