/***************************************************************************
			CladeSpecies.h
		-------------------
	begin	 : Mon Feb 12 15:23:53 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef CLADESPECIES_H
#define CLADESPECIES_H

#include "Species.h"
#include "CladeNode.h"
#include "Germe.h"

//! This class defines the base structure for a CladeSpecies object.
/**
*@author Jean-Philippe Doyon
*/

class CladeSpecies:public Species{

  friend std::ostream& operator << (std::ostream& output, const CladeSpecies& cladeSpecies );

 private:

  static int _classCounter;

  // (Species) Node on which it is associated
  const CladeNode* _cladeNode;

  // For the likelihood
  // This value depends on both children edges of this node 
  double _Dy; 

  // For clade species node only
  // Count of the number of ArtifGene reconcilied to this Species (essentialy for Reconciliation output)
  uint _slicedNodeCount;

  // Contains the Left/Right tree strings for each gene covered by the species
  // Left/Right strings are concatened as follow: Left + ";" + Right  
  TreeStringSet _treeStringSet;

  // Case A: Germe with 0 descendant at LEFT  and at least one at RIGHT
  // Case B: Germe with 0 descendant at RIGHT and at least one at LEFT
  uint _nbOfCaseA, _nbOfCaseB;

 public:

  static int classCounter(){ return _classCounter; };

  //Constructor and Destructor
  CladeSpecies( const State& state , const CladeNode* cladeNode );
  ~CladeSpecies(){ _classCounter--; }

  void clearForRedefine(){ 
    Species::clearForRedefine();
    _slicedNodeCount = 0;
  }

  // Access methods (Class attributes)
  const CladeNode* node() const { return (CladeNode*) (_node); }

  const SpeciesEdge* edge(Side s)  const;

  const Species* child(Side s) const;

  // Node type methods
  bool isCladeSpecies() const { return true; }

  // That is the maximal number of nodes (including ArtifNode) that can be covered by the species
  // Induced my the MaxRec
  uint maxNbOfClassId() const;

  // Sliced node count methods
  void incSlicedNodeCount()    { _slicedNodeCount++; }
  uint slicedNodeCount() const { return _slicedNodeCount; }
  void resetSlicedNodeCount()  { _slicedNodeCount = 0; }

  // Likelihood computation methods
  void compute_Dy();
  double Dy() const { return _Dy; }
  void constructAllTreeData( int upperClassId );
  void clear_H_Phi_Variables();

  // TreeStringSet methods

  void addTreeData( const Gene* gene );
  // Remove the tree data of the gene (children string) from 
  // the corresponding TreeStringSet
  void removeTreeData( const Gene* gene );

  // Gene tree random generation methods
  //        Recursive Birth-and-death (Arvestad et al.)
  void generateLeafGermes();
  //        Pure Birth-and-death
  void generateLeafGermes2();
  void deleteLostGermes();

  double probOfCase( const uint c ) const;

  uint nbOfLost() const { return _nbOfCaseA + _nbOfCaseB; }

  // Output methods
  string toString() const;
  void printNbOfCases(std::ostream& output) const;

  // Shortcut methods
  uint nbOfRec() const;
};

#endif
