#include "Edge.h"
#include "Tree.h"

uint Edge::_currentEdgeId = 0;
int  Edge::_classCounter = 0;

std::ostream& operator << (std::ostream& outputStream, const Edge& edge){
  outputStream << edge.toString();
  return outputStream;
}

Edge::Edge( const Node* fatherNode , const Node* childNode ):_fatherNode( fatherNode ) , _childNode( childNode ){
  _classCounter++;
  _edgeId = _currentEdgeId++;
  ( const_cast<Node*> ( _childNode ) )->setFatherEdge( this );
}

const Edge* Edge::fatherEdge() const{
  Exception( isRootEdge() , "Edge::fatherEdge() ----> is Root Edge" );
  return ( (const CladeNode*) (_fatherNode) )->fatherEdge();
}

const Edge* Edge::childEdge( Side side ) const{
  Exception( isLeafEdge() , "Edge::childEdge ----> is leaf edge" );
  const CladeNode* childNode = (const CladeNode*) ( _childNode );
  return childNode->edge( side );
}

const Tree& Edge::tree() const{
  Exception( _fatherNode == 0 , "Edge::tree() ---> fatherNode == 0" );
  return _fatherNode->tree();
}

uint Edge::layerId() const { 
  return _edgeId + tree().nbOfNodes(); 
}

string Edge::toString() const{
  string S = "(";

  if( isArtificial() )
    S += "A";
  else
    S += itos( _fatherNode->nodeId() );

  S += "," + itos( _childNode->nodeId() ) + ")";

  return S;
}

bool Edge::operator == ( const NodePair& nodePair ) const{
  return 
    ( _fatherNode->nodeId() == nodePair._id1 && _childNode->nodeId() == nodePair._id2 ) ||
    ( _fatherNode->nodeId() == nodePair._id2 && _childNode->nodeId() == nodePair._id1 );
}
