/***************************************************************************
			Edge.h
		-------------------
	begin	 : Mon Oct 16 13:35:49 EDT 2006
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef EDGE_H
#define EDGE_H

//! This class defines the base structure for a Edge object.
/**
*@author Jean-Philippe Doyon
*/

#include "Node.h"

struct NodePair{
  uint _id1;
  uint _id2;
  NodePair( uint id1, uint id2 ){
    _id1 = id1;
    _id2 = id2;
  }
};

class Edge{

  friend std::ostream& operator << (std::ostream& outputStream, const Edge& edge);

 private:
  static int _classCounter;

 protected:

  static uint _currentEdgeId;

  const Node* _fatherNode;
  const Node* _childNode;
  uint _edgeId;

 public:

  static int classCounter(){ return _classCounter; };

  static uint currentEdgeId() { return _currentEdgeId; }
  static void resetEdgeId()   { _currentEdgeId = 0; }

  // Constructor and Destructor
  Edge( const Node* fatherNode , const Node* childNode );
  ~Edge(){ _classCounter--; }

  // Access methods
  const Node* fatherNode() const { return _fatherNode; }
  const Node* childNode() const  { return _childNode; }

  bool isRootEdge() const { return _fatherNode->isRootNode(); }

  bool isLeafEdge() const { return _childNode->isLeafNode(); }
  
  const Edge* fatherEdge() const;

  const Edge* childEdge( Side side ) const;

  uint edgeId()     const { return _edgeId; }

  const Tree& tree() const;

  uint layerId() const;

  string toString() const;

  // Boolean method
  bool isArtificial() const {return _childNode->isRootNode(); };

  bool operator == ( const NodePair& nodePair ) const;


};

#endif
