#include "GeneEdge.h"
#include "Species.h"
#include "State.h"
#include "ArtifGene.h"

int GeneEdge::_classCounter = 0;

GeneEdge::GeneEdge( const State& state , const Edge* edge ):
  _state(state) , _edge(edge){
  _classCounter++;
}

const Gene* GeneEdge::fatherNode() const{
  return _state.getGene( _edge->fatherNode() );
}

const Gene* GeneEdge::childNode() const{
  return _state.getGene( _edge->childNode() );
}

// Let S, S1, S2 be the covering species of the new ArtifGene, the first and last on the Gene Edge
// The insertion of an ArtifGene onto the GeneEdge has 3 cases:
// Case A: it is the first ArtifGene
// Case B: S < S1, ArtifGene is inserted at the Beginning
// Case C: S2 < S, ArtifGene is inserted at the End
list< const ArtifGene* >::iterator GeneEdge::addArtifGene( const ArtifGene* artifGene ){
  // Case A
  if( _artifGeneList.empty() ){
    _artifGeneList.push_front( artifGene ); 
    return _artifGeneList.begin();
  }
  
  const Species* S  = artifGene->coveringSpecies();
  const Species* S1 = _artifGeneList.front()->coveringSpecies();
  const Species* S2 = _artifGeneList.back()->coveringSpecies();
  
  // Case B
  if( *S < *S1 ){
    _artifGeneList.push_front( artifGene ); 
    return _artifGeneList.begin();
  }
  // Case C
  if( *S2 < *S ){
    _artifGeneList.push_back( artifGene ); 
    return --( _artifGeneList.end() );
  }
  
  Exception( true , "GeneEdge::addArtifGene ---> Anormal Case" );
}

void GeneEdge::removeArtifGene( ArtifGene* artifGene ){
  Exception( _artifGeneList.empty() , "GeneEdge::removeArtifGene ---> artifGeneList is empty" );

  // Inverse operation of Case B
  if( _artifGeneList.front() == artifGene ){
    _artifGeneList.pop_front();
    return;
  }

  // Inverse operation of Case C
  if( _artifGeneList.back() == artifGene ){
    _artifGeneList.pop_back();
    return;
  }

  Exception( true , "GeneEdge::removeArtifGene ---> Anormal Case" );
}

const ArtifGene* GeneEdge::firstArtifGene() const{
  if( _artifGeneList.empty() )
    return 0;
  else
    return _artifGeneList.front();
}

const ArtifGene* GeneEdge::lastArtifGene() const{
  if( _artifGeneList.empty() )
    return 0;
  else
    return _artifGeneList.back();
}

// Gene Edge pass through the Species Edge <---> True
bool GeneEdge::isCoveredBy( const SpeciesEdge* speciesEdge ) const{
  const Species* species1;
  if( fatherNode()->isCoveredByASpecies() )
    species1 = fatherNode()->coveringSpecies();
  else
    species1 = ( (const RealGene*) ( fatherNode() ) )->coveringSpeciesEdge()->fatherNode();


  const Species* species2;
  if( childNode()->isCoveredByASpecies() )
    species2 = childNode()->coveringSpecies();
  else
    species2 = ( (const RealGene*) ( childNode() ) )->coveringSpeciesEdge()->childNode();

  return *species1 <= *( speciesEdge->fatherNode() ) && *( speciesEdge->childNode() ) <= *species2;
}
