/***************************************************************************
			GeneEdge.h
		-------------------
	begin	 : Wed Jan 10 10:16:05 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef GENEEDGE_H
#define GENEEDGE_H

//! This class defines the base structure for a GeneEdge object.
/**
*@author Jean-Philippe Doyon
*/

#include "Edge.h"
#include "SpeciesEdge.h"
#include "SlicedSubtree.h"

class ArtifGene;
class State;

class GeneEdge{

 private:

  static int _classCounter;

  const State& _state;
  const Edge* _edge;

  // The ArtifGene inserted on the GeneEdge
  // They have to be ordered, in the list, according to the ancestor relationship of their covering species
  list< const ArtifGene* > _artifGeneList;
  
 public:

  static int classCounter(){ return _classCounter; };

  // Constructor and destructor
  GeneEdge( const State& state , const Edge* _edge );
  
  ~GeneEdge(){ _classCounter--; }

  void clearForRedefine(){ _artifGeneList.clear(); }

  // Access methods
  const Edge* edge() const { return _edge; }

  const Gene* fatherNode() const;
  const Gene* childNode()  const;

  // Boolean Methods
  bool isLeftEdge() const{ return ( (const RealGene*) ( fatherNode() ) )->edge(LEFT) == this; }

  // Sliced methods
  list< const ArtifGene* >::iterator addArtifGene( const ArtifGene* artifGene );

  void removeArtifGene( ArtifGene* artifGene );

  const ArtifGene* firstArtifGene() const;
  const ArtifGene* lastArtifGene()  const;

  uint nbOfArtifGenes() const { return _artifGeneList.size(); }

  // Reconciliation methods
  bool isCoveredBy( const SpeciesEdge* speciesEdge ) const;
};

#endif
