#include "GeneTree.h"
#include "TreePair.h"


GeneTree::GeneTree( const TreePair& treePair , const ParamGeneTree& paramGeneTree )
  :Tree( treePair , paramGeneTree , false ){
}


string GeneTree::recVector_To_StringId( const vector<uint>& recVector ) const{
  string S;
  for( vector<Node*>::const_iterator node = _nodeVector.begin(); node != _nodeVector.end(); node++ )
    if( !(**node).isLeafNode() && !(**node).isRootNode() )
      S += itos( recVector[ (**node).nodeId() ] ) + " ";

  return string( S , 0 , S.size() - 1 );
}

void GeneTree::stringId_To_RecVector( const string& stringId , vector<uint>& recVector ) const{
  vector<string> stringVector;
  partitionString( stringId , stringVector );

  recVector.resize( nbOfNodes() );

  uint i = 0;

  for( vector<Node*>::const_iterator node = _nodeVector.begin(); node != _nodeVector.end(); node++ ){
    if( !(**node).isLeafNode() && !(**node).isRootNode() )
      recVector[ (**node).nodeId() ] = stoi( stringVector[i++] );
    else
      recVector[ (**node).nodeId() ] = _treePair.mappingFunction( *node )->nodeId();
  }
}

void GeneTree::writeToPrimeFile( ofstream& output , const Reconciliation& reconciliation ){
  // Depth First Search
  restartAll_DFS_Visited();

  list<CladeNode*> stackNode;
  stackNode.push_back( _root );

  output <<"(";

  while( !stackNode.empty() ){
    CladeNode* currentNode = stackNode.back();
    Node* nextNode         = currentNode->nextDFSChild();

    if( nextNode != 0 ){
      nextNode->setDFS_Visited( true );
      if( !nextNode->isLeftChild() )
	output <<",";

      if( !nextNode->isLeafNode() ){
	output <<"(";
	stackNode.push_back( (CladeNode*) nextNode );
      }
      else
	output << nextNode->primeString( reconciliation );
    }
    else{
      output <<")";
      output << currentNode->primeString( reconciliation );

      stackNode.pop_back();
    }
  }
  output <<";\n";
}
