#include "Germe.h"
#include "CladeSpecies.h"
#include "LeafSpecies.h"
#include "RandomTree.h"
#include "Types.h"

Germe::Germe( const Species* species ):
  _species(species){
}

void Germe::updateChildGermes( Side side ){
  vector<Germe*>& sideGerme = ( side == LEFT? _leftGermes: _rightGermes );

  // Count the number of non lost germes
  uint nbOfGermes = 0;
  for( vector<Germe*>::iterator germe = sideGerme.begin(); germe != sideGerme.end(); germe++ ){
    if( !(**germe).isLost() )
      nbOfGermes++;
  }

  // Create new germes vector with non lost germe
  //      and delete lost germes
  vector<Germe*> tmpVector( nbOfGermes );
  int i = 0;
  for( vector<Germe*>::iterator germe = sideGerme.begin(); germe != sideGerme.end(); germe++ )
    if( (**germe).isLost() )
      delete *germe;
    else{
      tmpVector[i] = *germe;
      i++;
    }

  sideGerme.clear();
  sideGerme = tmpVector;
}

void Germe::addLeafGermes( uint leftNb , uint rightNb ){
  Exception( _species->isLeafNode() , "Germe::addLeafGermes ---> species is a leaf" );
  const CladeSpecies* species = (const CladeSpecies*) (_species);

  _leftGermes.resize( leftNb );
  for( uint i = 0; i < _leftGermes.size(); i++ ){
    _leftGermes[i] = new Germe( species->child( LEFT ) );
    const_cast<Species*> ( ( species->child( LEFT ) ) )->addGerme( _leftGermes[i] );
  }

  _rightGermes.resize( rightNb );
  for( uint i = 0; i < _rightGermes.size(); i++ ){
    _rightGermes[i] = new Germe( species->child( RIGHT ) );
    const_cast<Species*> ( ( species->child( RIGHT ) ) )->addGerme( _rightGermes[i] );
  }
}

void Germe::transformTreeString( const string& origin , string& dest , const vector<Germe*>& germes ){
  for( int i = 0; i < origin.size(); i++ ){
    if( isdigit( origin[i] ) ){
      uint id = stoi( readTaxaId( origin , i ) );
      dest += germes[ id ]->treeString();
    }
    else{
      if( origin[i] == '(' )
	dest += "[" + ctos(EDGE_TAG) + itos( _species->nodeId() ) + "]";
      dest += origin[i];
    }
  }
}

void Germe::computeTreeString(){
  if( _species->isLeafNode() )
    _treeString = itos( ( (const LeafSpecies*) ( _species ) )->node()->specieId() );
  else{
    string origin1 = randomTree( _leftGermes.size() );
    string dest1;
    transformTreeString( origin1 , dest1 , _leftGermes );

    string origin2  = randomTree( _rightGermes.size()  );
    string dest2;
    transformTreeString( origin2 , dest2 , _rightGermes );

    Exception( dest1.empty() && dest2.empty() , "Germe::computeTreeString() ---> both string are empty" );

    if( dest1.empty() )
      _treeString += dest2;
    else
      if( dest2.empty() )
	_treeString += dest1;
      else{
	_treeString += "[" + itos( _species->nodeId() ) + "]";
	_treeString += "(" + dest1 + "," + dest2 + ")";
      }
  }
}
