/***************************************************************************
			Germe.h
		-------------------
	begin	 : Tue Aug 28 15:49:40 PDT 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef GERME_H
#define GERME_H

#include "StdClass.h"
#include "Species.h"

//! This class defines the base structure for a Germe object.
/**
*@author Jean-Philippe Doyon
*/

class Germe{

 private:
  vector<Germe*> _leftGermes;
  vector<Germe*> _rightGermes;
  const Species* _species;
  string _treeString;

 public:

  Germe( const Species* species );

  ~Germe(){}

  bool isLost() const{ 
    if( !_species->isCladeSpecies() )
      return false;
    else
      return _leftGermes.empty() && _rightGermes.empty(); 
  }

  // Update left/right Germes according to lost germes
  void updateChildGermes( Side side );

  // Access methods
  string treeString() const { return _treeString; }

  // Create and Add the indicated number of leaf germes for both left and right children
  void addLeafGermes( uint leftNb , uint rightNb );

  // Case A: species is a Clade
  //     Choose a random sliced subtree for left/right germes
  //     Compute the sliced subtree according to theses trees and left/right germes
  // Case B: species is a Leaf
  //     Tree String is the corresponding species Id
  void computeTreeString();

 private:
  void transformTreeString( const string& origin , string& dest , const vector<Germe*>& germes );


};

#endif
