/***************************************************************************
			LeafNode.h
		-------------------
	begin	 : Fri Jun  9 13:58:40 EDT 2006
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef LEAFNODE_H
#define LEAFNODE_H

//! This class defines the base structure for a LeafNode object.
/**
*@author Jean-Philippe Doyon
*/

#include "Node.h"
#include "StdClass.h"
#include "toolBox.h"

class LeafNode:public Node{

 private:

  static int _classCounter;

  static uint _currentLeafId;

  const uint _specieId;
  uint _leafId;
  UintSet _cladeSet; // Will contain only one unsigned integer

  // This is used when M( r(G) ) != r(S)
  // A new extant gene is inserted in G s.t. this constraint is respected
  // This insertion does not change any combinatorial aspect of the rec. space
  // The leaf is not considered when V() is computed
  bool _fictiveLeaf;

 public:

  static int classCounter(){ return _classCounter; };
  static void resetCurrentLeafId() { _currentLeafId = 0; }

  // Constructor and Destructor
  LeafNode( uint depth , uint specieId , const Tree& tree , const CladeNode* father = 0 );

  ~LeafNode(){ _classCounter--; }

  // Access methods
  uint specieId() const { return _specieId; }
  uint leafId()   const { return _leafId; }
  const UintSet& cladeSet() const { return _cladeSet; }

  bool withoutEdge() const { Exception( true , "LeafNode::withoutEdge ----> unavailable function call" ); return true;}

  const Edge* edge( Side s ) const{ return 0; }

  void setFictive( bool b ) { _fictiveLeaf = b; }

  bool fictiveLeaf() const { return _fictiveLeaf; }

  // Boolean methods
  bool isLeafNode() const { return true; }

  // Cell set method
  void defineCellSet();
  Cell* highestCell() const{ return _lowestCell; }

  // Normal form methods
  bool isFirstInNormForm( const Node* rightNode ) const;

  // Output methods
  string primeString( const Reconciliation& reconciliation ) const;

};

#endif
