/***************************************************************************
			SlideSet.h
		-------------------
	begin	 : Fri May 18 17:34:38 PDT 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef SLIDESET_H
#define SLIDESET_H

//! This class defines the base structure for a SlideSet object.
/**
*@author Jean-Philippe Doyon
*/

#include "State.h"

class SlideSet{

  friend std::ostream& operator << ( std::ostream& output , const SlideSet& slideSet );

 private:

  // All possibles slides are conserved in a vector
  vector<Slide>    _possibleSlides;
  
  // Bijection: nodeId <---> SlidePos (up/down)
  vector<SlidePos> _slidesPositions;

  // The size of the Set
  // It corresponds to the position next to the last slide in the vector possibleSlides
  uint _size;

 public:

  SlideSet(){};

  void clear(){
    _possibleSlides.clear();
    _slidesPositions.clear();
    _size = 0;
  }

  // Construct the SlideSet according the reconciliation represented by the state
  void initialize( const State& state );

  // Access methods
  const vector<Slide>&  possibleSlides() const { return _possibleSlides; }
  const vector<SlidePos>& slidesPositions() const { return _slidesPositions; }

  // Information methods
  int size() const { return _size; }
  bool isEmpty() const { return _size == 0; }

  // Set methods
  void addSlide      ( const RealGene* realGene , DIRECTION dir );
  void removeSlide   ( const RealGene* realGene , DIRECTION dir );
  bool containsSlide ( const RealGene* realGene , DIRECTION dir ) const;

  // Return the slide at position id in _possiblesSlides
  const Slide& slideAt( int id ) const;

  // Choose a slide of the set such that it is
  //    - a random selection
  //    - different to the inverse slide of S 
  //      (the last slide performed in the MCMC; to prevent mcmc loop)
  // The slide is removed from the set and all other slides that will be
  // inconsistant with the new reconciliation
  Slide randomSlide( const Slide& last , const State& state ) const;

  void update( const Slide& next , const State& state );

 private:
  void specDup( const Slide& next , const State& state );
  void dupDup ( const Slide& next , const State& state );

};

#endif
