#include "Species.h"
#include "SlicedSubtree.h"
#include "State.h"
#include "LeafSpecies.h"
#include "Germe.h"
#include "CladeSpecies.h"

int Species::_classCounter = 0;

Species::Species( const State& state , const Node* node ):
  _state( state ) , _node( node ){
  _classCounter++;
}

Species::~Species(){ 
  _classCounter--; 

  for( list<Germe*>::iterator germe = _germesList.begin(); germe != _germesList.end(); germe++ )
    delete *germe;
}

const Species* Species::father() const{
  Exception( isRootNode() , "Species::fatherNode() ---> rootNode " );
  return fatherEdge()->fatherNode();
}


const SpeciesEdge* Species::fatherEdge() const{ 
  return _state.getSpeciesEdge( _node->fatherEdge() ); 
}

void Species::removeCoveredGene( const Gene* gene ){ 
  _geneList.remove( const_cast<Gene*> (gene) ); 
}


bool Species::operator == ( const Species& species ) const{
  return nodeId() == species.nodeId();
}

bool Species::operator != ( const Species& species ) const{
   return !( *this == species );
}

bool Species::operator <  ( const Species& species ) const{
  return tree().ancestorMatrixValue( this->node() , species.node() ); 
}

bool Species::operator <= ( const Species& species ) const{
   return *this == species || *this < species;
}

bool Species::operator >  ( const Species& species ) const{
   return species < *this;
}

bool Species::operator >= ( const Species& species ) const{
   return species <= *this;
}

// std::ostream& operator << (std::ostream& output, const Species& species ){
//   output << species.nodeId();
//   return output;
// }

// For each germe that belongs to this species, 
//        compute the treeString
void Species::computeAllGermeStrings() const{
  for( list<Germe*>::const_iterator germe = _germesList.begin(); germe != _germesList.end(); germe++ )
    (**germe).computeTreeString();
}

// string Species::primeSpeciesString() const{
//   string s;
//   if( !isCladeSpecies() )
//     s += itos( ( (const LeafSpecies&) (*this) ).node()->specieId() );

//   /*
//     This has been disable since Species ids require to be those of primetv
//     s += "[&&PRIME ID=" + itos( nodeId() )+ "]";
//   */

//   return s;
// }

void Species::printGermesList( std::ostream& output ) const{
  output<< "\n\nSpecies: " << toString();
  output<< "\n\tNb of germes: " << _germesList.size();

  if( !isLeafNode() )
    ( (const CladeSpecies*) (this) )->printNbOfCases( output );

}
