/***************************************************************************
			Species.h
		-------------------
	begin	 : Sun Jan  7 18:51:58 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef NODEVALUES_H
#define NODEVALUES_H

//! This class defines the base structure for a Species object.
/**
*@author Jean-Philippe Doyon
*/

#include "Node.h"
#include "SpeciesEdge.h"
#include "Gene.h"
#include "TreeStringSet.h"

class State;
class Germe;

class Species{

  //  friend std::ostream& operator << (std::ostream& output, const Species& species );

 protected:
  static int _classCounter;

  // The corresponding state
  const State& _state; 

  // (Species) Node on which it is associated
  const Node* _node;

  // Reconciliation attributes
  // Let u be a Gene and y be the current Species
  //      u in Gamma(y) <----> u in List
  list< Gene* > _geneList;

  // The germe that are mapped on this Species
  list<Germe*> _germesList;

 public:

  static int classCounter(){ return _classCounter; };

  // Constructor and Destructor
  Species( const State& state , const Node* node );

  virtual ~Species();


  virtual void clearForRedefine(){ _geneList.clear(); }


  // Access methods (Class attributes)
  const Species* father() const;
  const SpeciesEdge* fatherEdge() const;

  virtual const Node* node()      const = 0;
  const list< Gene* >& geneList() const { return _geneList; }

  const list<Germe*>& germesList() const { return _germesList; }

  // Node type methods
  virtual bool isCladeSpecies() const = 0;

  virtual uint maxNbOfClassId() const = 0;

  bool isRootNode() const { return _node->isRootNode(); }
  bool isLeafNode() const { return _node->isLeafNode(); }

  // Shortcut methods  
  const Tree& tree() const { return _node->tree(); }
  uint nodeId()      const { return _node->nodeId(); }
  const UintSet& cladeSet() const { return _node->cladeSet(); }

  // Reconciliation methods
  void addCoveredGene   ( const Gene* gene ){ _geneList.push_back( const_cast<Gene*> (gene) ); }
  void removeCoveredGene( const Gene* gene );

  // Likelihood computation methods
  //  virtual void constructAllTreeData( int nb ) = 0;

  // Binary comparaison operators
  // x < y iff x is an ancestor of y
  bool operator == ( const Species& species ) const;
  bool operator != ( const Species& species ) const;
  bool operator <  ( const Species& species ) const;
  bool operator <= ( const Species& species ) const;
  bool operator >  ( const Species& species ) const;
  bool operator >= ( const Species& species ) const;
  bool disjointTo  ( const Species& species ) const;

  // Gene tree random generation methods
  void computeAllGermeStrings() const;
  void addGerme( Germe* germe ){ _germesList.push_back( germe ); }
  uint nbOfGermes() const { return _germesList.size(); }

  // Output methods
  void printGermesList( std::ostream& output ) const;
  virtual string toString() const = 0;


};

#endif
