#include "SpeciesTree.h"

SpeciesTree::SpeciesTree( const TreePair& treePair , const ParamSpeciesTree& paramSpeciesTree )
  :Tree( treePair , paramSpeciesTree , true ){
  /////////////////////////
  // WHY isASpeciesTree is FALSE????
  /////////////////////////

  // Build the specie id 2 leaf vector
  _specieId_2_LeafVector.resize( nbOfLeaves() );

  vector<Node*>::const_iterator iterNode;
  for( iterNode = _nodeVector.begin(); iterNode != _nodeVector.end(); iterNode++ )
    if( (**iterNode).isLeafNode() ){
      const LeafNode* leafNode = (const LeafNode*) *iterNode;   
      _specieId_2_LeafVector[ leafNode->specieId() ] = const_cast<LeafNode*> (leafNode);
    }

  computeAllCladeSet();
}

const LeafNode* SpeciesTree::specieId_2_LeafNode( uint specieId ) const{
  Exception( _specieId_2_LeafVector.size() == 0                             , "_specieId_2_LeafVectorTree::specieId_2_LeafNode ---> vector is empty " );
  Exception( vectorIdOutOfBound( specieId , _specieId_2_LeafVector.size() ) , "_specieId_2_LeafVectorTree::specieId_2_LeafNode ---> specie id out of bound " );

  return _specieId_2_LeafVector[ specieId ];
}

// std::ostream& operator << (std::ostream& output, const SpeciesTree& speciesTree ){
//   return output;
// }

void SpeciesTree::printConsistency( std::ostream& output , const BirthDeathParam& birthDeathParam ) const{
  if( nbOfEdges() != birthDeathParam.size() ){
    output<< "\nClade with default edge values";
    for( uint i = 0; i < nbOfEdges(); i++ ){
      const UintSet& cladeSet = getEdge(i)->childNode()->cladeSet();
      if( !birthDeathParam.edgeValuesIsDefined( cladeSet ) )
	output<<"\n\t"<<cladeSet;
    }
    output<<endl;
    PAUSE();
  }
}
