/***************************************************************************
			StateSet.h
		-------------------
	begin	 : Fri Jul 27 18:20:19 PDT 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef STATESET_H
#define STATESET_H

#include "ObjectSet.h"
#include "State.h"
#include "StateInfo.h"
#include "StateLayer.h"

#include "OrthoParam.h"
#include "TreePair.h"

//! This class defines the base structure for a StateSet object.
/**
*@author Jean-Philippe Doyon
*/

// The state set is implemented as follows:
// Each layer is defined by vector of size equals to the number of SpeciesNode + SpeciesEdge
// 1st layer: 1st gene, 2st layer:2st gene, etc
// For N st layer, a vector position exists <---> the corresponding species covers the N st gene

class StateSet:public ObjectSet{

 private:
  uint _nbOfGenes;
  uint _totalCount;
  const TreePair& _treePair;

 public:

  // Constructor and Destructor
  StateSet( const TreePair& treePair ): _treePair( treePair ) { _nbOfGenes = 0; _totalCount = 0; }
  ~StateSet() { objectSetClear(); }

  // Initialization of the State set
  void initialize( uint speciesTreeSize , uint nbOfGenes );

  // Access methods
  uint size() const { return stateInfoList().size(); }

  const StateLayer* rootLayer() const { return (const StateLayer*) _rootLayer; }
  list< const StateInfo* >& stateInfoList() const { return ( list< const StateInfo* >& ) ( _objInfoList ); }

  void computePostProb();
  void computeApproxPostProb();

  uint nbOfNonNullState() const;

  uint totalCount() const { return _totalCount; }
  StateInfo* addState( const vector<uint>& recVector , const string& stateInfo_Id = string() );
  
  const StateInfo& getStateInfo( const State& state ) const;
  const StateInfo& getStateInfo( const vector<uint>& recVector ) const;

  void checkStateConsistency( const State& state ) const;

  // State Ordered List (list ordered in increasing order of true post prob)
  void orderStateInfoList();
  // True <----> stateInfo has the highest true post prob
  bool isBestState( const StateInfo& stateInfo) const { return stateInfo.id() == stateInfoList().back()->id(); }

  // State space graph methods
  void computeEdgesVector( const TreePair& treePair , const OrthoParam& orthoParam );

  // Output methods
  void writeAllState( std::ostream& output , const TreePair& treePair , const BirthDeathParam& birthDeathParam , double minPostProb ) const;
  void writeAcceptanceProb( std::ostream& output , const OrthoParam& param ) const;

  void print( std::ostream& output , const OrthoParam& orthoParam ) const;


};

#endif
