#include "TreeData.h"
#include "MathClass.h"

int TreeData::_classCounter = 0;

TreeData::TreeData( uint classId ){
  _classCounter++;

  _string = classId == 0? "0": itos( classId );

  _size   = 1;
  _nbAuto = _nbLabel = 0;
  _automorphique = _distinctLabel = false;
  _upperClassId  = classId;
  _betaFct       = 1;
}

TreeData::TreeData( const TreeData& treeData1 , const TreeData& treeData2){
  _classCounter++;

  _size         = treeData1.size() + treeData2.size();
  _upperClassId = max( treeData1.upperClassId() , treeData2.upperClassId() );

  _betaFct = MathClass::combination( treeData1.size() - 1 , treeData1.size() + treeData2.size() - 2 ) * treeData1.betaFct() * treeData2.betaFct();
  _nbAuto  = treeData1.nbAuto()  + treeData2.nbAuto();
  _nbLabel = treeData1.nbLabel() + treeData2.nbLabel();

  // Case 1: |T_1| > |T_2|, then S <--- (S_1,S_2)
  if( treeData1.size() > treeData2.size() ){
    _string = "(" + treeData1.String() + "," + treeData2.String() + ")";
    _automorphique = _distinctLabel = false;
    return;
  }

  // Case 2: |T_1| < |T_2|, then S <--- (S_2,S_1)
  if( treeData1.size() < treeData2.size() ){
    _string = "(" + treeData2.String() + "," + treeData1.String() + ")";
    _automorphique = _distinctLabel = false;
    return;
  }

  // Case 3: |T_1| = |T_2|
  // We need to compute wich one of the trees (T1/T2) is larger than the others in order to
  // define Tree String based on S1 and S2.
  _distinctLabel = false;
  _automorphique = true;
  bool tree1IsLarger = false;

  int i = 0; int j = 0;
  while( i < treeData1.String().size() && j < treeData2.String().size() ){

    if( treeData1.String()[i] == '(' &&  treeData2.String()[j] != '(' ){
      _string = "(" + treeData1.String() + "," + treeData2.String() + ")";
      _automorphique = false;
      return;
    }

    if( treeData1.String()[i] != '(' &&  treeData2.String()[j] == '(' ){
      _string = "(" + treeData2.String() + "," + treeData1.String() + ")";
      _automorphique = false;
      return;
    }

    // These exceptions are only flag to prevent for unexpected Tree Data Strings
    // TO DO: PROOF THAT THEY ARE IMPOSSIBLE
    Exception(  isdigit( treeData1.String()[i] ) && !isdigit( treeData2.String()[j] ) , "TreeData:: impossible case" );
    Exception( !isdigit( treeData1.String()[i] ) &&  isdigit( treeData2.String()[j] ) , "TreeData:: impossible case" );

    if( isdigit( treeData1.String()[i] ) && isdigit( treeData2.String()[j] ) ){
      // TO ADD: consider the fact of id longer than 1 character
      uint id1 = stoi( readTaxaId( treeData1.String() , i ) ); //ctoi( treeData1.String()[i] );
      uint id2 = stoi( readTaxaId( treeData2.String() , j ) ); //ctoi( treeData2.String()[j] );
      _distinctLabel = _distinctLabel? true: id1 != id2;
      tree1IsLarger = tree1IsLarger?  true: id1 > id2;
    }
    i++; j++;
  }

  _nbAuto++; // u is automorphique
  if( _distinctLabel )
    _nbLabel++;

  if( tree1IsLarger )
    _string = "(" + treeData1.String() + "," + treeData2.String() + ")";
  else
    _string = "(" + treeData2.String() + "," + treeData1.String() + ")";
}


std::ostream& operator << ( std::ostream& output , const TreeData& data ){
  output << data.size();

  output <<"\t";
  output << data.automorphique();

  output <<"\t";
  if( data.automorphique() )
    output << data.distinctLabel();

  output <<"\t\t" << data.upperClassId();
  output <<"\t"   << data.betaFct();
  output <<"\t"   << data.String();

  return output;
}
