/***************************************************************************
			TreeString.h
		-------------------
	begin	 : Thu Feb  8 22:07:28 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef TREEDATA_H
#define TREEDATA_H

//! This class defines the base structure for a TreeData object.
/**
*@author Jean-Philippe Doyon
*/

#include "StdClass.h"
#include "toolBox.h"

class TreeData{

  friend std::ostream& operator << ( std::ostream& output , const TreeData& data );

 private:
  static int _classCounter;

  // Canonical representation of the tree
  string _string;
  
  // The size is defined by the number of leaves
  uint _size;

  // A node u is automorphique iff. T_u_1 = T_u_2 (without regarding leaf labels)
  // The number of automorphique nodes in T_u (including u)
  uint _nbAuto;
  bool _automorphique;

  // A node u has distinct label iff. u is automorphique AND T_u_1 \neq T_u_2 considering leaf labels
  // The number of automorphique nodes with distinct label in T_u (including u)
  bool _nbLabel;
  bool _distinctLabel;

  // The upper class id contained in it
  uint _upperClassId;

  uint _betaFct;

 public:

  static int classCounter(){ return _classCounter; }

  // Constructor 1: used for (leaf) subtree
  TreeData( uint classId );

  // Constructor 2: used for (tree,tree)
  TreeData( const TreeData& treeData1 , const TreeData& treeData2);

  // Destructor
  ~TreeData(){ _classCounter--; }

  const string& String() const { return _string; }
  uint size()            const { return _size; }
  uint nbAuto()          const { return _nbAuto; }
  bool nbLabel() const { return _nbLabel; }
  bool automorphique() const { return _automorphique; }
  bool distinctLabel() const { return _distinctLabel; }

  uint upperClassId() const { return _upperClassId; }
  uint betaFct()      const { return _betaFct; }
  
};

#endif
