#include "ValuationValues.h"

ValuationValues::ValuationValues(){
  _dup = _lost = 0;
  _mut = lastUint;
  _lik = 0;
  _postProb = 0;
  _nmcDist.resize( Types::nbOfRecName() , 0 );
}

ValuationValues::ValuationValues( const ValuationValues& origin ){
  _dup  = (uint ) ( origin.value( Dup )  );
  _lost = (uint ) ( origin.value( Lost ) );
  _mut  = (uint ) ( origin.value( Mut ) );

  if( Types::nbOfValuations() >= 4 )
    _lik  = origin.value( Lik );

  if( Types::nbOfValuations() >= 5 )
    _postProb = origin.value( PostProb );

  _nmcDist.resize( Types::nbOfRecName() , 0 );
}

void ValuationValues::operation( uint val , uint value , int sign ){
  Valuation valuation = (Valuation) (val);
  Exception( valuation == Mut      , " ValuationValues::add ---> mutation is forbiden" );
  Exception( valuation == Lik      , " ValuationValues::add ---> likelihood is forbiden" );
  Exception( valuation == PostProb , " ValuationValues::add ---> likelihood is forbiden" );

  Exception( sign != 1 && sign != -1 , "ValuationValues::add ---> sign is either -1 or +1 ");

  if( valuation == Dup )
    _dup += (sign) * value;
    
  if( valuation == Lost )
    _lost += (sign) * value;
}

double ValuationValues::value( uint val ) const{
  Exception( !( 0 <= val && val < Types::nbOfValuations() ) , "ValuationValues::value ---> val is not valid" );
  Valuation valuation = (Valuation) (val);

  // Duplication
  if( valuation == Dup )
    return _dup;

  // Lost
  if( valuation == Lost )
    return _lost;

  // Mutation
  if( valuation == Mut ){
//     if( _mut != lastUint )
//       return _mut;
//     else
      return _dup + _lost;
  }
  
  // Likelihood
  if( valuation == Lik )
    return _lik;

  // Post. prob.
  if( valuation == PostProb )
    return _postProb;
}

void ValuationValues::setValue( uint val , double value ){
  Exception( !( 0 <= val && val < Types::nbOfValuations() ) , "ValuationValues::setValue ---> val is not valid" );
  Valuation valuation = (Valuation) (val);

  // Duplication
  if( valuation == Dup )
    _dup = (uint) (value);

  // Lost
  if( valuation == Lost )
    _lost = (uint) (value);

  // Mutation
  if( valuation == Mut )
    _mut = (uint) (value);

  // Likelihood
  if( valuation == Lik )
     _lik = value;

  // Post. prob.
  if( valuation == PostProb )
    _postProb = value;

}

void ValuationValues::setWorstValues(){
  // Minimisation criteria
  _dup  = lastUint;
  _lost = lastUint;
  _mut  = lastUint;

  // Maximisation criteria
  _lik      = 0;
  _postProb = 0;
}

bool ValuationValues::hasWorstValue( uint val , double V ) const{
  Exception( !( 0 <= val && val < Types::nbOfValuations() ) , "ValuationValues::hasWorstValue ---> val is not valid" );
  Valuation valuation = (Valuation) (val);
  if( valuation == Lik || valuation == PostProb )
    return value( val ) < V;
  else
    return value( val ) > V;
}

bool ValuationValues::hasBetterValue( uint val , double V ) const{
  Exception( !( 0 <= val && val < Types::nbOfValuations() ) , "ValuationValues::hasBetEqValue ---> val is not valid" );

  Valuation valuation = (Valuation) (val);
  if( valuation == Lik || valuation == PostProb )
    return value( val ) > V;
  else
    return value( val ) < V;
}

bool ValuationValues::hasBetEqValue( uint val , double V ) const{
  Exception( !( 0 <= val && val < Types::nbOfValuations() ) , "ValuationValues::hasBetEqValue ---> val is not valid" );
  Valuation valuation = (Valuation) (val);
  if( valuation == Lik || valuation == PostProb )
    return value( val ) >= V;
  else
    return value( val ) <= V;

}

// NMC distances
void ValuationValues::setNMCDist( REC_NAME recName , uint dist ){
  string msg = "setNMCDist ---> recName out of bound:";
  msg += itos( recName ) + " , " + itos( _nmcDist.size() );
  Exception( vectorIdOutOfBound( recName , _nmcDist.size() ) , msg );

  _nmcDist[ recName ] = dist;
}

uint ValuationValues::nmcDist( REC_NAME recName ) const{
  string msg = "nmcDist ---> recName out of bound:";
  msg += itos( recName ) + " , " + itos( _nmcDist.size() );
  return _nmcDist[ recName ];
}

const ValuationValues& ValuationValues::operator = ( const ValuationValues& right ){
  Exception( true , "ValuationValues::operator = ---> this operator doesn't work!!!!" );
  _dup  = (uint ) ( right.value( Dup )  );
  _lost = (uint ) ( right.value( Lost ) );
  _mut  = (uint ) ( right.value( Mut ) );

  if( Types::nbOfValuations() == 4 )
    _lik = right.value( Lik );

  if( Types::nbOfValuations() == 5 )
    _postProb  = right.value( PostProb );

  return *this;
}

std::ostream& operator << ( std::ostream& output , const ValuationValues& valuationValues ){
  output<< "(";
  for( uint val = 0; val < Types::nbOfValuations(); val++ )
    output << valuationValues.value( val ) << " ";
  output<< ")";
  return output;
}
