/***************************************************************************
			ValuationValues.h
		-------------------
	begin	 : Sat May 17 16:11:12 EDT 2008
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef VALUATIONVALUES_H
#define VALUATIONVALUES_H

//! This class defines the base structure for a ValuationValues object.
/**
*@author Jean-Philippe Doyon
*/

#include "StdClass.h"
#include "toolBox.h"
#include "Types.h"

class ValuationValues{

friend std::ostream& operator << ( std::ostream& output , const ValuationValues& valuationValues );

 private:

 // Discrete value: min value are better
 uint _dup;
 uint _lost;
 uint _mut;
  
 // Likelihood value: max value are better
 double _lik;

 // Posterior probabilities in {0 ... 1}
 //        That is  lik / likSum
 double _postProb;

 // NMC distance between MinRec, RealRec, LikRec
 // Vector or size 3
 vector<uint> _nmcDist;

 public:

  ValuationValues();

  ValuationValues( const ValuationValues& origin );

  ~ValuationValues(){}

  // Only for dup/lost costs
  void add( uint val , uint value = 1 ){ operation( val , value , +1 ); }

  void sub( uint val , uint value = 1 ){ operation( val , value , -1 ); }

  void addSub( uint val , OperatorDir opDir , uint value = 1 ){
    if( opDir == Add )
      add( val , value );
    else
      sub( val , value );
  }

  double value( uint val ) const;
  void setValue( uint val , double value );

  void setWorstValues();

  bool hasWorstValue( uint val , double V ) const;
  bool hasBetEqValue( uint val , double V ) const;
  bool hasBetterValue( uint val , double V ) const;

  // NMC distances
  void setNMCDist( REC_NAME recName , uint dist );
  uint nmcDist   ( REC_NAME recName ) const;

  const ValuationValues& operator = ( const ValuationValues& right );


 private:

  void operation( uint val , uint value , int sign );
  

};

#endif
