#include "OrthoParam.h"
#include "MathClass.h"

const string GENE_TREE     = "geneTree";
const string SPECIES_TREE  = "speciesTree";
const string EDGE_VALUES   = "edgeValues";

const char SLIDE_DOWN   ='D';
const char SLIDE_UP     ='U';

const string DIR_NAME_TAG = "DirName";
const string NGEN_TAG = "NGen";
const string SAMPLE_FREQ_TAG = "SampleFreq";
const string DIAGN_FREQ_TAG = "DiagnFreq";
const string SWAP_FREQ_TAG = "SwapFreq";
const string TEMP_TAG = "Temp";

string paramToTag( Param param ){
  if( param == DIR_NAME )
    return "DirName";

  if( param == NRUN )
    return "Nrun";

  if( param == NGEN )
    return "Ngen";

  if( param == SAMPLE_FREQ )
    return "SampleFreq";

  if( param == DIAGN_FREQ)
    return "DiagnFreq";

  if( param == SWAP_FREQ)
    return "SwapFreq";

  if( param == TEMP )
    return "Temp";

  if( param == NCHAINS)
    return "Nchains";

  if( param == INPUT_POST_PROB )
    return "InputPostProb";

  if( param == LOG_PRINT )
    return "LogFile";

  if( param == LOG_FILE_NAME )
    return "LogFileName";

  if( param == DOT_FILE )
    return "DotFile";

  if( param == PRIMES )
    return "Prime";

  if( param == OUTPUTREC )
    return "OutputRec";

  if( param == PRINT )
    return "Print";

  if( param == CHAINID )
    return "ChainId";

  if( param == SEED )
    return "Seed";

  if( param == FIRST_REC )
    return "FirstRec";

  if( param == OUTPUT_POST_PROB )
    return "OutputPostProb";

  if( param == MOVE_TYPE )
    return "MoveType";

  if( param == PRINT_FREQ )
    return "PrintFreq";

  if( param == BURNIN ) 
    return "Burnin";

  if( param == BURNIN_FRAC )
    return "BurninFrac";

  if( param == REL_BURNIN )
    return "RelBurnin";

  if( param == STATE_ID )
    return "StateId";

  if( param == EXACT_LIK )
    return "ExactLik";

}

Param tagToParam( const string& tag ){
  if( tag.compare( paramToTag( DIR_NAME ) ) == 0 )
    return DIR_NAME;

  if( tag.compare( paramToTag( NRUN ) ) == 0 )
    return NRUN;

  if( tag.compare( paramToTag( NGEN ) ) == 0 )
    return NGEN;

  if( tag.compare( paramToTag( SAMPLE_FREQ ) ) == 0 )
    return SAMPLE_FREQ;

  if( tag.compare( paramToTag( DIAGN_FREQ ) ) == 0 )
    return DIAGN_FREQ;

  if( tag.compare( paramToTag( SWAP_FREQ ) ) == 0 )
    return SWAP_FREQ;

  if( tag.compare( paramToTag( TEMP ) ) == 0 )
    return TEMP;

  if( tag.compare( paramToTag( NCHAINS ) ) == 0 )
    return NCHAINS;

  if( tag.compare( paramToTag( INPUT_POST_PROB ) ) == 0 )
    return INPUT_POST_PROB;

  if( tag.compare( paramToTag( LOG_PRINT ) ) == 0 )
    return LOG_PRINT;

  if( tag.compare( paramToTag( LOG_FILE_NAME ) ) == 0 )
    return LOG_FILE_NAME;

  if( tag.compare( paramToTag( DOT_FILE ) ) == 0 )
    return DOT_FILE;

  if( tag.compare( paramToTag( PRIMES ) ) == 0 )
    return PRIMES;

  if( tag.compare( paramToTag( OUTPUTREC ) ) == 0 )
    return OUTPUTREC;

  if( tag.compare( paramToTag( PRINT ) ) == 0 )
    return PRINT;

  if( tag.compare( paramToTag( CHAINID ) ) == 0 )
    return CHAINID;

  if( tag.compare( paramToTag( SEED ) ) == 0 )
    return SEED;

  if( tag.compare( paramToTag( FIRST_REC ) ) == 0 )
    return FIRST_REC;

  if( tag.compare( paramToTag( OUTPUT_POST_PROB ) ) == 0 )
    return OUTPUT_POST_PROB;

  if( tag.compare( paramToTag( MOVE_TYPE ) ) == 0 )
    return MOVE_TYPE;

  if( tag.compare( paramToTag( PRINT_FREQ ) ) == 0 )
    return PRINT_FREQ;

  if( tag.compare( paramToTag( BURNIN ) ) == 0 )
    return BURNIN;

  if( tag.compare( paramToTag( BURNIN_FRAC ) ) == 0 )
    return BURNIN_FRAC;

  if( tag.compare( paramToTag( REL_BURNIN ) ) == 0 )
    return REL_BURNIN;

  if( tag.compare( paramToTag( STATE_ID ) ) == 0 )
    return STATE_ID;

  if( tag.compare( paramToTag( EXACT_LIK ) ) == 0 )
    return EXACT_LIK;

  Exception( true , string( "tagToParam --> invalid tag: \"" ) + tag + "\"");
}

string OrthoParam::paramString( Param param ) const{

  if( param == INPUT_POST_PROB ){
    Exception( !paramIntValue( INPUT_POST_PROB ) , "OrthoParam::paramString ----> InputPostProb is disactived" );
    string fileName = paramString( DIR_NAME ) + string( "inputPostProb" );
    Exception( !fileExists( fileName ) , "OrthoParam::paramString ----> file does not exist" );
    return fileName;
  }
  else
    return _paramVector[ param ]; 
}

int OrthoParam::paramIntValue( Param param ) const{
  Exception( param == TEMP , "OrthoParam::paramIntValue ---> invalid parameters" );

  string value = _paramVector[ param ];

  // Default value
  if( value.empty() )
    return defaultValue( param );
  else
    return stoi( value );
}

double OrthoParam::paramDoubleValue( Param param ) const{
  Exception( param != TEMP && param != BURNIN_FRAC , "OrthoParam::paramDoubleValue ---> invalid parameters" );
  string value = _paramVector[ param ];
  return stod( value );
}

int OrthoParam::defaultValue( Param param ) const{
  if( param == INPUT_POST_PROB || param == OUTPUTREC || param == DIAGN_FREQ )
    return 0;

  if( param == NRUN || param == SAMPLE_FREQ || param == OUTPUT_POST_PROB )
    return 1;

  return 0;

  Exception( true , "OrthoParam::defaultValue ---> default value undefined " + paramToTag( param ) );
}

OrthoParam::OrthoParam( int argc, char* argv[] ):
  _paramVector( NB_PARAM ){

  ////////////////////////
  // OPTIONS
  // Log file 

  ////////////////////////

  string inputFile = string( argv[1] , strlen( argv[1] ) );

  ifstream inputStream( inputFile.c_str() );

  string buffer;
  getline( inputStream , buffer );
  while( !inputStream.eof() ){
    if( buffer.size() > 0 && buffer[0] != '#' ){
      vector< string > stringVector;
      partitionString( buffer , stringVector , BLANK );
      string tag   = stringVector[0];
      string value = stringVector[1];

      if( tagToParam( tag ) == FIRST_REC && isdigit( value ) )
	value = stringVector2String( stringVector , 1 );

      _paramVector[ tagToParam( tag ) ] = value;
    }
    getline( inputStream , buffer );
  }

  inputStream.close();

  // Define default value
  if( paramIntValue( NCHAINS ) == 1 ){
    _paramVector[ SWAP_FREQ ] = _paramVector[ NGEN ];
    if( paramString( CHAINID ).empty() )
      _paramVector[ CHAINID ] = "0";
  }
  if( paramString( SEED ).empty() ){
    int seed = MathClass::genSeed();
    _paramVector[ SEED ] = itos( seed );
  }

  _logStream = new ofstream( paramString( LOG_FILE_NAME).c_str() , std::ios::out );
  
  string speciesTree = readFirstLine( paramString( DIR_NAME ) + SPECIES_TREE );
  string geneTree    = readFirstLine( paramString( DIR_NAME ) +  GENE_TREE );
  string edgeValues  = paramString( DIR_NAME ) + EDGE_VALUES;

  
  _paramSpeciesTree.initialize( speciesTree );
  _paramGeneTree.initialize( _paramSpeciesTree , geneTree );


  // Birth Death Param
  if( fileExists( edgeValues ) )
    _birthDeathParam.initialize( _paramSpeciesTree , edgeValues , false );
  else
    _birthDeathParam.initialize();

  // Burnin value
  if( paramIntValue( REL_BURNIN ) == 1 )
    _paramVector[ BURNIN ] = itos( paramIntValue( NGEN ) * paramDoubleValue( BURNIN_FRAC ) );
}

OrthoParam::OrthoParam( const OrthoParam& orthoParam , uint chainId ):
  _paramVector( NB_PARAM ){
  for( int i = 0; i < _paramVector.size(); i++ )
    _paramVector[ i ] = orthoParam.paramVector()[i];

  _paramVector[ NRUN ]     = itos( 1 );
  _paramVector[ NCHAINS ]  = itos( 1 );
  _paramVector[ CHAINID ]  = itos( chainId );

  _paramVector[ LOG_FILE_NAME ] += "-" + itos( chainId );

}

OrthoParam::~OrthoParam(){
  if( paramIntValue( PRINT ) && paramIntValue( LOG_PRINT ) ){
    _logStream->close();
    delete _logStream;
  }
}


string OrthoParam::help(){
  string s = "\n\nOptions:";
  return s;
}

std::ostream& operator << ( std::ostream& output , const OrthoParam& param ){
  output<<"\n\tSpecies tree:\t" << param.paramSpeciesTree().treeString2();
  output<<"\n\tGene tree:\t"   << param.paramGeneTree().treeString2();

  return output;
}

void OrthoParam::print( ostream& stream ) const{
  stream<<"\n";
  for( int i = 0; i < _paramVector.size(); i++ ){
    Param param = (Param) (i);
    stream<< paramToTag( param )  <<"\t" << _paramVector[ param ] <<"\n";
  }
}
