/***************************************************************************
			Parameters.h
		-------------------
	begin	 : Wed Feb 28 09:10:05 EST 2007
	copyright: (C) 2005 by Universite De Montreal
	email	 :  doyonjea@iro.umontreal.ca
***************************************************************************/
#ifndef ORTHOPARAM_H
#define ORTHOPARAM_H

//! This class defines the base structure for a Parameters object.
/**
*@author Jean-Philippe Doyon
*/

#include "StdClass.h"
#include "toolBox.h"
#include "SlideClasses.h"
#include "ParamSpeciesTree.h"
#include "ParamGeneTree.h"
#include "BirthDeathParam.h"

const string NMC_MOVE   = "NMC";
const string RANDOM_REC = "R";

// Constant for the RandomData algorithm
const uint DIST_VECTOR_SIZE = 100;

const double MAX_DIFF_RATIO = 0.0000000001;

const double MIN_PRECISION = 0.000000001;

enum Option{PRIME , OUTPUT, LOGFILE , DOTFILE , CHAIN_LENGTH , COMPUTE_MAPPING_PROB , CHAIN_TEMP , CHAIN_ID , INPUT_REC , OUTPUT_REC , SLIDE , RANDOM_FIRST , POST_PROB , PERFORM_EXPLORATION ,
	    LIK , CHECK_STATE_SET , MAX_DEPTH , OUTPUT_STEP };

///////////////////////////
// Options for MCMC
// See MrBayes 3.1 Manual (p.16) for description
enum Param{ DIR_NAME , NRUN , NGEN , SAMPLE_FREQ , DIAGN_FREQ , SWAP_FREQ , TEMP , NCHAINS , INPUT_POST_PROB ,  OUTPUT_POST_PROB ,
	    LOG_PRINT , LOG_FILE_NAME , DOT_FILE , PRIMES , OUTPUTREC , CHAINID , SEED , FIRST_REC , MOVE_TYPE , PRINT_FREQ , 
	    BURNIN , BURNIN_FRAC , REL_BURNIN ,
	    STATE_ID , EXACT_LIK ,
	    PRINT  }; // THE LAST ONE HAS TO BE PRINT

const int NB_PARAM = PRINT + 1;

const string logFileName = "orthoBayesCluster.log";

// Parameters type:
// Integer: NRUN, NGEN, SAMPLE_FREQ, DIAGN_FREQ, SWAP_FREQ, NCHAINS, CHAINID, SEED
// Double: TEMP
// String: 

string getOptionString( Option option );

int optionDefaultValue( Option option );

class OrthoParam{

  friend std::ostream& operator << ( std::ostream& output , const OrthoParam& param );

 private:

  vector<string> _paramVector;

  ofstream* _logStream; // Defined iff when _logFile is true

  ParamSpeciesTree _paramSpeciesTree;
  ParamGeneTree    _paramGeneTree;
  BirthDeathParam  _birthDeathParam;

  // One file for each chain
  vector< string > _recFileNames;

  // Only when nchain = 1
  vector< uint > _recVector;

 public:

  const vector<string>& paramVector() const { return _paramVector; }

  string paramString( Param param ) const;
  int    paramIntValue ( Param param ) const;

  double paramDoubleValue( Param param ) const;

  int    defaultValue( Param param ) const;

  string fileName() const;

  // Constructor
  OrthoParam( int argc, char* argv[] );
  OrthoParam( const OrthoParam& orthoParam , uint chainId );

  ~OrthoParam();

  // Output ostream
  std::ostream& Ostream( int i ) const{ return ( paramIntValue( LOG_PRINT )? *_logStream: cerr ); }

  // Access to class variables
  const string& recFileName( int chainId ) { return _recFileNames[ chainId ]; }

  // Define class variables
  void setRecVector( const vector< uint >& recVector ){ _recVector = recVector; }

  // Access to output options

  const ParamSpeciesTree& paramSpeciesTree() const { return _paramSpeciesTree; }
  const ParamGeneTree&    paramGeneTree()    const { return _paramGeneTree; }
  const BirthDeathParam&  birthDeathParam()  const { return _birthDeathParam; }

  // Access to MCMC Options
  // Input reconciliation option

  const vector<uint>& recVector() const { return _recVector; }
  bool inputRecOn() const { return !_recVector.empty(); }

  // First Reconciliation 
  bool firstRecIsLowest() const { return _paramVector[FIRST_REC] == "L"; }
  bool firstRecIsRandom() const { return _paramVector[FIRST_REC] == "R"; }
  bool firstRecIsGiven()  const { return !firstRecIsLowest() && !firstRecIsRandom(); }
  
  // Help String
  static string help();
  void print( ostream& stream ) const;

 private:
  void defineParameter( const string& buffer );

};

#endif
